from acai_aws.apigateway.resolver.modes.base import BaseModeResolver
from acai_aws.apigateway.exception import ApiException


class DirectoryModeResolver(BaseModeResolver):

    INIT_FILE = '__init__.py'

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.__handler_path = self.importer.clean_path(kwargs['handlers'])

    def _get_file_and_import_path(self, request_path):
        # ensure previous lookups don't leave behind path/dynamic state
        self.reset()
        split_path = self.get_request_path_as_list(request_path)
        route_path = self.__get_route_path(split_path)
        file_path = self.__handler_path + self.importer.file_separator + route_path
        import_path = self.get_import_path(file_path)
        return file_path, import_path

    def __get_route_path(self, split_path):
        file_tree = self.importer.get_handlers_file_tree()
        self.__get_import_path_file_tree(split_path, 0, file_tree)
        return f'{self.importer.file_separator}'.join(self.import_path)

    def __get_import_path_file_tree(self, split_path, split_index, file_tree):
        if split_index < len(split_path):
            part = split_path[split_index]
            if part == '':
                possible_directory = None
                possible_file = self.INIT_FILE
            else:
                possible_directory = part.replace('-', '_')
                possible_file = f'{possible_directory}.py'
            if possible_directory in file_tree:
                self.__handle_directory_path_part(possible_directory, split_path, split_index, file_tree)
            elif possible_file in file_tree:
                self.__handle_file_path_part(possible_file, split_path, split_index, file_tree)
            elif file_tree.get('__dynamic_files') and file_tree['__dynamic_files']:
                self.__handle_dynamic_path_part(split_path, split_index, file_tree)
            else:
                raise ApiException(code=404, message='route not found')

    def __handle_directory_path_part(self, possible_directory, split_path, split_index, file_tree):
        self.append_import_path(possible_directory)
        if split_index+1 < len(split_path):
            file_leaf = self.determine_which_file_leaf(file_tree, possible_directory)
            self.__get_import_path_file_tree(split_path, split_index+1, file_leaf)
        else:
            self.append_import_path(self.INIT_FILE)

    def __handle_file_path_part(self, possible_file, split_path, split_index, file_tree):
        self.append_import_path(possible_file)
        file_leaf = self.determine_which_file_leaf(file_tree, possible_file)
        self.__get_import_path_file_tree(split_path, split_index+1, file_leaf)

    def __handle_dynamic_path_part(self, split_path, split_index, file_tree):
        file_part = list(file_tree['__dynamic_files'])[0]
        self.append_import_path(file_part)
        if '.py' not in file_part and split_index+1 == len(split_path):
            self.append_import_path(self.INIT_FILE)
        file_leaf = self.determine_which_file_leaf(file_tree, file_part)
        self.has_dynamic_route = True
        self.dynamic_parts[split_index] = split_path[split_index]
        self.__get_import_path_file_tree(split_path, split_index+1, file_leaf)



from acai_aws.common.json_helper import JsonHelper
from acai_aws.base.record import BaseRecord


class Record(BaseRecord):

    @property
    def message_id(self):
        return self._record.get('messageId')

    @property
    def batch_item_identifier(self):
        message_id = self.message_id
        return {'itemIdentifier': message_id} if message_id is not None else None

    @property
    def receipt_handle(self):
        return self._record.get('receiptHandle')

    @property
    def body(self):
        return JsonHelper.decode(self._record.get('body'))

    @property
    def operation(self):
        return self.UNKNOWN

    @property
    def message_attributes(self):
        return self._record.get('messageAttributes')

    @property
    def attributes(self):
        self._attributes = self._record.get('attributes', {}).copy()
        for key in self.message_attributes:
            attr = self.message_attributes[key]
            # Support both camelCase (AWS Lambda) and PascalCase (LocalStack/SDKs)
            value = attr.get('stringValue') or attr.get('StringValue')
            if not value:
                value = attr.get('binaryValue') or attr.get('BinaryValue')
            self._attributes[key] = value
        return self._attributes

    @property
    def region(self):
        return self._record.get('awsRegion')

    @property
    def source_arn(self):
        return self._record.get('eventSourceARN')

    @property
    def source(self):
        return self._record.get('eventSource')

    @property
    def md5_of_body(self):
        return self._record.get('md5OfBody')

    def __str__(self):
        return str({
            'message_id': self.message_id,
            'receipt_handle': self.receipt_handle,
            'body': self.body,
            'operation': self.operation,
            'message_attributes': self.message_attributes,
            'attributes': self.attributes,
            'region': self.region,
            'source_arn': self.source_arn,
            'source': self.source,
            'md5_of_body': self.md5_of_body
        })



from acai_aws.apigateway.requirements import requirements


@requirements(
    required_query=['email'],
    response_codes={
        200: 'User exists',
        204: 'User does not exist'
    }
)
def get(_, response):
    response.body = {'exists': True}
    return response


@requirements(
    required_response={
        'type': 'object',
        'required': ['id'],
        'properties': {
            'id': {'type': 'string'}
        }
    },
    response_codes={
        200: 'Resource found',
        204: 'Resource not found'
    }
)
def post(_, response):
    response.body = {'id': '123'}
    return response



from acai_aws.apigateway.requirements import requirements

@requirements(required_route='/triple/{x}/{y}/{z}')
def post(request, response):
    response.body = {'pattern_triple_coordinates': request.path_params}
    return response



from acai_aws.apigateway.router import Router


router = Router(
    base_path='your-service/v1',
    handlers='api/handlers',
    schema='api/openapi.yml'
)
router.auto_load()

def route(event, context):
    return router.route(event, context)


from acai_aws.apigateway.requirements import requirements

@requirements(required_route='/user/{user_id}/item/{item_id}')
def get(request, response):
    response.body = {'pattern_user_id_item_id': request.path_params}
    return response



from acai_aws.apigateway.requirements import requirements

@requirements(required_route='/dynamic/{id}')
def post(request, response):
    response.body = {'directory_dynamic': True}
    return response



import json
import unittest

from acai_aws.alb.router import Router

from tests.mocks.alb import mock_request


class ALBRouterTest(unittest.TestCase):
    """Smoke tests confirming alb.Router routes ALB events through the apigateway
    Router internals while applying ALB-specific request/response handling."""

    maxDiff = None
    base_path = 'unit-test/v1'
    handler_path = 'tests/mocks/apigateway/router/directory_handlers'

    def test_basic_directory_routing_works(self):
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_path,
        )
        result = router.route(mock_request.get_basic_post(), None)
        self.assertEqual(200, result['statusCode'])
        self.assertEqual('200 OK', result['statusDescription'])
        self.assertFalse(result['isBase64Encoded'])
        self.assertDictEqual(
            {'router_directory_basic': {'body_key': 'body_value'}},
            json.loads(result['body']),
        )

    def test_routing_decodes_base64_body_end_to_end(self):
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_path,
        )
        result = router.route(mock_request.get_basic_post_base64(), None)
        self.assertEqual(200, result['statusCode'])
        self.assertEqual('200 OK', result['statusDescription'])
        self.assertDictEqual(
            {'router_directory_basic': {'body_key': 'body_value'}},
            json.loads(result['body']),
        )

    def test_routing_supports_dynamic_path_parameters(self):
        event = mock_request.get_dynamic_event(
            path='unit-test/v1/user/1',
            method='get',
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_path,
        )
        result = router.route(event, None)
        self.assertEqual(200, result['statusCode'])
        self.assertDictEqual(
            {'directory_user_id': {'user_id': '1'}},
            json.loads(result['body']),
        )

    def test_routing_supports_multiple_dynamic_segments(self):
        event = mock_request.get_dynamic_event(
            path='unit-test/v1/triple/1/2/3',
            method='post',
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_path,
        )
        result = router.route(event, None)
        self.assertEqual(200, result['statusCode'])
        self.assertDictEqual(
            {'directory_triple_coordinates': {'x': '1', 'y': '2', 'z': '3'}},
            json.loads(result['body']),
        )

    def test_unknown_route_returns_404_with_status_description(self):
        event = mock_request.get_dynamic_event(
            path='unit-test/v1/does-not-exist',
            method='get',
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_path,
        )
        result = router.route(event, None)
        self.assertEqual(404, result['statusCode'])
        self.assertEqual('404 Not Found', result['statusDescription'])

    def test_routing_can_opt_out_of_cors(self):
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_path,
            cors=False,
        )
        result = router.route(mock_request.get_basic_post(), None)
        self.assertNotIn('Access-Control-Allow-Origin', result['headers'])


if __name__ == '__main__':
    unittest.main()



import abc

from acai_aws.apigateway.resolver.importer import ResolverImporter


class BaseModeResolver(abc.ABC):

    def __init__(self, **kwargs):
        self.importer = ResolverImporter(handlers=kwargs['handlers'])
        self.base_path = self.importer.clean_path(kwargs['base_path'])
        self.has_dynamic_route = False
        self.file_tree_climbed = True
        self.dynamic_parts = {}
        self.import_path = []

    @abc.abstractmethod
    def _get_file_and_import_path(self, request_path):
        raise NotImplementedError

    def load_importer_files(self):
        self.importer.get_handlers_file_tree()

    def get_endpoint_module(self, request):
        file_path, import_path = self._get_file_and_import_path(request.path)
        return self.importer.import_module_from_file(file_path, import_path)

    def get_import_path(self, relative_file_path):
        return relative_file_path.replace(self.importer.file_separator, '.').replace('.py', '')

    def get_request_path_as_list(self, request_path):
        base_path = request_path.replace(self.base_path, '')
        clean_base = self.importer.clean_path(base_path)
        return clean_base.split('/')

    def determine_which_file_leaf(self, file_tree, file_branch):
        if file_tree.get(file_branch) and file_tree[file_branch] != '*':
            self.file_tree_climbed = True
            return file_tree[file_branch]
        self.file_tree_climbed = False
        return file_tree

    def append_import_path(self, path_part):
        if self.file_tree_climbed:
            self.import_path.append(path_part)

    def reset(self):
        self.has_dynamic_route = False
        self.dynamic_parts = {}
        self.file_tree_climbed = True
        self.import_path = []



import json


def get_aws_websocket_example():
    return {
        'version': '2.0',
        'routeKey': 'ANY /nodejs-apig-function-1G3XMPLZXVXYI',
        'rawPath': '/default/nodejs-apig-function-1G3XMPLZXVXYI',
        'rawQueryString': '',
        'cookies': [
            's_fid=7AABXMPL1AFD9BBF-0643XMPL09956DE2',
            'regStatus=pre-register'
        ],
        'headers': {
            'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
            'accept-encoding': 'gzip, deflate, br',
            'accept-language': 'en-US,en;q=0.9',
            'content-length': '0',
            'host': 'r3pmxmplak.execute-api.us-east-2.amazonaws.com',
            'sec-fetch-dest': 'document',
            'sec-fetch-mode': 'navigate',
            'sec-fetch-site': 'cross-site',
            'sec-fetch-user': '?1',
            'upgrade-insecure-requests': '1',
            'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.132 Safari/537.36',
            'x-amzn-trace-id': 'Root=1-5e6722a7-cc56xmpl46db7ae02d4da47e',
            'x-forwarded-for': '205.255.255.176',
            'x-forwarded-port': '443',
            'x-forwarded-proto': 'https'
        },
        'requestContext': {
            'accountId': '123456789012',
            'apiId': 'r3pmxmplak',
            'domainName': 'r3pmxmplak.execute-api.us-east-2.amazonaws.com',
            'domainPrefix': 'r3pmxmplak',
            'http': {
                'method': 'GET',
                'path': '/default/nodejs-apig-function-1G3XMPLZXVXYI',
                'protocol': 'HTTP/1.1',
                'sourceIp': '205.255.255.176',
                'userAgent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.132 Safari/537.36'
            },
            'requestId': 'JKJaXmPLvHcESHA=',
            'routeKey': 'ANY /nodejs-apig-function-1G3XMPLZXVXYI',
            'stage': 'default',
            'time': '10/Mar/2020:05:16:23 +0000',
            'timeEpoch': 1583817383220
        },
        'isBase64Encoded': True
    }


def get_aws_example():
    return {
        'resource': '/',
        'path': '/',
        'httpMethod': 'GET',
        'headers': {
            'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
            'accept-encoding': 'gzip, deflate, br',
            'accept-language': 'en-US,en;q=0.9',
            'cookie': 's_fid=7AAB6XMPLAFD9BBF-0643XMPL09956DE2; regStatus=pre-register',
            'Host': '70ixmpl4fl.execute-api.us-east-2.amazonaws.com',
            'sec-fetch-dest': 'document',
            'sec-fetch-mode': 'navigate',
            'sec-fetch-site': 'none',
            'upgrade-insecure-requests': '1',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.132 Safari/537.36',
            'X-Amzn-Trace-Id': 'Root=1-5e66d96f-7491f09xmpl79d18acf3d050',
            'X-Forwarded-For': '52.255.255.12',
            'X-Forwarded-Port': '443',
            'X-Forwarded-Proto': 'https'
        },
        'multiValueHeaders': {
            'accept': [
                'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9'
            ],
            'accept-encoding': [
                'gzip, deflate, br'
            ],
            'accept-language': [
                'en-US,en;q=0.9'
            ],
            'cookie': [
                's_fid=7AABXMPL1AFD9BBF-0643XMPL09956DE2; regStatus=pre-register;'
            ],
            'Host': [
                '70ixmpl4fl.execute-api.ca-central-1.amazonaws.com'
            ],
            'sec-fetch-dest': [
                'document'
            ],
            'sec-fetch-mode': [
                'navigate'
            ],
            'sec-fetch-site': [
                'none'
            ],
            'upgrade-insecure-requests': [
                '1'
            ],
            'User-Agent': [
                'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.132 Safari/537.36'
            ],
            'X-Amzn-Trace-Id': [
                'Root=1-5e66d96f-7491f09xmpl79d18acf3d050'
            ],
            'X-Forwarded-For': [
                '52.255.255.12'
            ],
            'X-Forwarded-Port': [
                '443'
            ],
            'X-Forwarded-Proto': [
                'https'
            ]
        },
        'queryStringParameters': None,
        'multiValueQueryStringParameters': None,
        'pathParameters': None,
        'stageVariables': None,
        'requestContext': {
            'resourceId': '2gxmpl',
            'resourcePath': '/',
            'httpMethod': 'GET',
            'extendedRequestId': 'JJbxmplHYosFVYQ=',
            'requestTime': '10/Mar/2020:00:03:59 +0000',
            'path': '/Prod/',
            'accountId': '123456789012',
            'protocol': 'HTTP/1.1',
            'stage': 'Prod',
            'domainPrefix': '70ixmpl4fl',
            'requestTimeEpoch': 1583798639428,
            'requestId': '77375676-xmpl-4b79-853a-f982474efe18',
            'identity': {
                'cognitoIdentityPoolId': None,
                'accountId': None,
                'cognitoIdentityId': None,
                'caller': None,
                'sourceIp': '52.255.255.12',
                'principalOrgId': None,
                'accessKey': None,
                'cognitoAuthenticationType': None,
                'cognitoAuthenticationProvider': None,
                'userArn': None,
                'userAgent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.132 Safari/537.36',
                'user': None
            },
            'domainName': '70ixmpl4fl.execute-api.us-east-2.amazonaws.com',
            'apiId': '70ixmpl4fl'
        },
        'body': json.dumps({'body_key': 'body_value'}),
        'isBase64Encoded': False
    }


def get_basic():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'x-authorizer-key': 'SOME KEY',
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': 'unit-test/v1/basic',
        'pathParameters': {
            'proxy': 'hello'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'GET',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': json.dumps({'body_key': 'body_value'})
    }


def get_basic_offline():
    return {
        'isOffline': True,
        'headers': {
            'x-api-key': 'SOME-KEY',
            'x-authorizer-key': 'SOME KEY',
            'content-type': 'application/json'
        },
        'path': 'unit-test/v1/basic',
        'pathParameters': {
            'proxy': 'hello'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'GET',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': json.dumps({'body_key': 'body_value'})
    }


def get_basic_form():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'x-authorizer-key': 'SOME KEY',
            'content-type': 'application/x-www-form-urlencoded'
        },
        'path': 'unit-test/v1/basic',
        'pathParameters': {
            'proxy': 'hello'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'GET',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': 'param1=data1&param2=data2&param3=data3'
    }


def get_basic_xml():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'x-authorizer-key': 'SOME KEY',
            'content-type': 'application/xml'
        },
        'path': 'unit-test/v1/basic',
        'pathParameters': {
            'proxy': 'hello'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'GET',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': '<root xmlns="http://defaultns.com/" xmlns:a="http://a.com/" xmlns:b="http://b.com/"><x>1</x><a:y>2</a:y><b:z>3</b:z></root>'
    }


def get_basic_raw():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'x-authorizer-key': 'SOME KEY'
        },
        'path': 'unit-test/v1/basic',
        'pathParameters': {
            'proxy': 'hello'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'GET',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': 'unit-test'
    }


def get_basic_graphql():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'x-authorizer-key': 'SOME KEY',
            'content-type': 'application/graphql'
        },
        'path': 'unit-test/v1/basic',
        'pathParameters': {
            'proxy': 'hello'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'GET',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': '{players{name}}'
    }


def basic_graphql_variables():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'x-authorizer-key': 'SOME KEY',
            'content-type': 'application/graphql'
        },
        'path': 'unit-test/v1/basic',
        'pathParameters': {
            'proxy': 'hello'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'GET',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': json.dumps(
            {
                'query': 'query GreetingQuery ($arg1: String) { hello (name: $arg1) { value } }',
                'operationName': 'GreetingQuery',
                'variables': {'arg1': 'Timothy'}
            }
        )
    }


def get_basic_nested():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'x-authorizer-key': 'SOME KEY',
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': 'unit-test/v1/nested-1/nested-2/basic',
        'pathParameters': {
            'proxy': 'hello'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'GET',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': json.dumps({'body_key': 'body_value'})
    }


def get_basic_init():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'x-authorizer-key': 'SOME KEY',
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': 'unit-test/v1/home/',
        'pathParameters': {
            'proxy': 'hello'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'GET',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': json.dumps({'body_key': 'body_value'})
    }


def get_dynamic():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'x-authorizer-key': 'SOME KEY',
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': 'unit-test/v1/dynamic/1',
        'pathParameters': {
            'proxy': 'hello'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'GET',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': json.dumps({'body_key': 'body_value'})
    }


def get_bad_route():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'x-authorizer-key': 'SOME KEY',
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': 'unit-test/v1/bad/route',
        'pathParameters': {
            'proxy': 'hello'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'GET',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': json.dumps({'body_key': 'body_value'})
    }


def get_basic_post():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'x-authorizer-key': 'SOME KEY',
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': 'unit-test/v1/basic',
        'pathParameters': {
            'proxy': 'hello'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'POST',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': json.dumps({'body_key': 'body_value'})
    }

def get_basic_post_bad_method():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'x-authorizer-key': 'SOME KEY',
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': 'unit-test/v1/basic',
        'pathParameters': {
            'proxy': 'hello'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'PUT',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': json.dumps({'body_key': 'body_value'})
    }


def get_dynamic_post():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'x-authorizer-key': 'SOME KEY',
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': 'unit-test/v1/dynamic/1',
        'pathParameters': {},
        'resource': '/{proxy+}',
        'httpMethod': 'POST',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': json.dumps({'body_key': 'body_value'})
    }


def get_bad_dynamic_post():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'x-authorizer-key': 'SOME KEY',
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': 'unit-test/v1/bad_dynamic/1',
        'pathParameters': {},
        'resource': '/{proxy+}',
        'httpMethod': 'POST',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': json.dumps({'body_key': 'body_value'})
    }

def get_bad_dynamic_get():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'x-authorizer-key': 'SOME KEY',
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': 'unit-test/v1/bad_dynamic/1',
        'pathParameters': {},
        'resource': '/{proxy+}',
        'httpMethod': 'GET',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': json.dumps({})
    }


def get_no_dynamic_post():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'x-authorizer-key': 'SOME KEY',
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': 'unit-test/v1/no_dynamic/1',
        'pathParameters': {},
        'resource': '/{proxy+}',
        'httpMethod': 'POST',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': json.dumps({'body_key': 'body_value'})
    }


def get_raised_exception_post():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'x-authorizer-key': 'SOME KEY',
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': 'unit-test/v1/raise-exception',
        'pathParameters': {
            'proxy': 'hello'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'POST',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': json.dumps({'body_key': 'body_value'})
    }


def get_unhandled_exception_post():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'x-authorizer-key': 'SOME KEY',
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': 'unit-test/v1/unhandled-exception',
        'pathParameters': {
            'proxy': 'hello'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'POST',
        'queryStringParameters': {
            'name': 'me'
        },
        'body': json.dumps({'body_key': 'body_value'})
    }


def get_basic_for_validation():
    return {
        'headers': {
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': 'unit-test/v1/basic',
        'pathParameters': {
            'proxy': 'basic'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'GET',
        'queryStringParameters': {
            'email': 'some-email'
        },
        'body': json.dumps({})
    }


def get_basic_passing_for_required_body_validation():
    return {
        'headers': {
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': 'unit-test/v1/basic',
        'pathParameters': {
            'proxy': 'basic'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'GET',
        'queryStringParameters': {},
        'body': json.dumps(
            {
                'id': 3,
                'email': 'some@email.com',
                'active': True,
                'favorites': ['anime', 'video games', 'basketball'],
                'notification_config': {
                    'marketing': False,
                    'transactions': True
                }
            }
        )
    }


def get_basic_failing_for_required_body_validation():
    return {
        'headers': {
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': 'unit-test/v1/basic',
        'pathParameters': {
            'proxy': 'basic'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'GET',
        'queryStringParameters': {},
        'body': json.dumps(
            {
                'email': 'some@email.com',
                'active': True,
                'favorites': ['anime', 'video games', 'basketball'],
                'notification_config': {
                    'marketing': False,
                    'transactions': True
                }
            }
        )
    }


def get_dynamic_event(**kwargs):
    return {
        'headers': kwargs.get('headers', {}),
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': kwargs.get('path', 'unit-test/v1/basic'),
        'pathParameters': {
            'proxy': kwargs.get('proxy', 'basic')
        },
        'resource': '/{proxy+}',
        'httpMethod': kwargs.get('method', 'GET').upper(),
        'queryStringParameters': kwargs.get('query', {}),
        'body': json.dumps(kwargs.get('body', {}))
    }


def get_auto_validated_data():
    return {
        'headers': {
            'x-api-key': 'some-key',
            'cookie': 's_fid=7AAB6XMPLAFD9BBF-0643XMPL09956DE2; regStatus=pre-register',
            'Host': 'localhost:3000',
            'Content-Type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'domainName': 'localhost:3000',
            'protocol': 'HTTP/1.1',
            'authorizer': {
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': '/unit-test/v1/auto',
        'pathParameters': {
            'proxy': 'auto'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'POST',
        'queryStringParameters': {},
        'body': json.dumps(
            {
                'test_id': 'abc123',
                'object_key': {
                    'key': 'value'
                },
                'array_number': [1, 2, 3],
                'array_objects': [
                    {
                        'array_string_key': 'string_value',
                        'array_number_key': 0
                    }
                ]
            }
        )
    }


def get_auto_validated_data_fails():
    return {
        'headers': {
            'x-api-key': 'some-key',
            'cookie': 's_fid=7AAB6XMPLAFD9BBF-0643XMPL09956DE2; regStatus=pre-register',
            'Host': 'localhost:3000',
            'Content-Type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'domainName': 'localhost:3000',
            'protocol': 'HTTP/1.1',
            'authorizer': {
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': '/unit-test/v1/auto',
        'pathParameters': {
            'proxy': 'auto'
        },
        'resource': '/{proxy+}',
        'httpMethod': 'POST',
        'queryStringParameters': {},
        'body': json.dumps(
            {
                'object_key': {
                    'key': 'value'
                },
                'array_number': [1, 2, 3],
                'array_objects': [
                    {
                        'array_string_key': 'string_value',
                        'array_number_key': 0
                    }
                ]
            }
        )
    }


def get_triple_post():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'x-authorizer-key': 'SOME KEY',
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': 'unit-test/v1/triple/1/2/3',
        'pathParameters': {},
        'resource': '/{proxy+}',
        'httpMethod': 'POST',
        'queryStringParameters': {},
        'body': json.dumps({'body_key': 'body_value'})
    }

def get_dynamic_nested_request_get(path=''):
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'x-authorizer-key': 'SOME KEY',
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'path': f'unit-test/v1/{path}',
        'pathParameters': {},
        'resource': '/{proxy+}',
        'httpMethod': 'POST',
        'queryStringParameters': {},
        'body': {}
    }


def get_request_with_missing_fields():
    return {
        'headers': {
            'x-api-key': 'SOME-KEY',
            'content-type': 'application/json'
        },
        'requestContext': {
            'resourceId': 't89kib',
            'authorizer': {
                'x-authorizer-key': 'SOME KEY',
                'principalId': '9de3f415a97e410386dbef146e88744e',
                'integrationLatency': 572,
            }
        },
        'pathParameters': {},
        'body': {}
    }



from acai_aws.apigateway.router import Router as ApigatewayRouter
from acai_aws.alb.request import Request
from acai_aws.alb.response import Response


class Router(ApigatewayRouter):
    request_class = Request
    response_class = Response



import unittest

from acai_aws.apigateway.request import Request
from acai_aws.apigateway.response import Response
from acai_aws.apigateway.resolver.modes.mapping import MappingModeResolver
from acai_aws.apigateway.exception import ApiException

from tests.mocks.apigateway import mock_request


class MappingModeResolverTest(unittest.TestCase):
    basic_request = mock_request.get_basic()
    nested_request = mock_request.get_basic_nested()
    init_request = mock_request.get_basic_init()
    dynamic_request = mock_request.get_dynamic()
    bad_route_request = mock_request.get_bad_route()
    base_path = 'unit-test/v1'
    handler_mapping_preferred = {
        'basic': 'tests/mocks/apigateway/resolver/mapping_handlers/basic.py'
    }
    handler_mapping_leading_slashes = {
        '/basic': '/tests/mocks/apigateway/resolver/mapping_handlers/basic.py'
    }
    handler_mapping_base_path_leading_slashes = {
        '/unit-test/v1/basic': '/tests/mocks/apigateway/resolver/mapping_handlers/basic.py'
    }
    handler_mapping_base_path_mismatch_slashes = {
        'unit-test/v1/basic': '/tests/mocks/apigateway/resolver/mapping_handlers/basic.py'
    }
    handler_mapping = {
        '/': 'tests/mocks/apigateway/resolver/mapping_handlers/basic.py',
        'basic': 'tests/mocks/apigateway/resolver/mapping_handlers/basic.py',
        'home': 'tests/mocks/apigateway/resolver/mapping_handlers/home/__init__.py',
        'nested-1/nested-2/basic': 'tests/mocks/apigateway/resolver/mapping_handlers/nested_1/nested_2/basic.py',
        'dynamic/{id}': 'tests/mocks/apigateway/resolver/mapping_handlers/dynamic/_id.py'
    }

    def setUp(self):
        self.mapping_resolver = MappingModeResolver(base_path=self.base_path, handlers=self.handler_mapping)

    def test_get_file_and_import_path_module_with_preferred_mapping(self):
        mapping_resolver = MappingModeResolver(base_path=self.base_path, handlers=self.handler_mapping_preferred)
        request = Request(self.basic_request)
        file_path, import_path = mapping_resolver._get_file_and_import_path(request.path)
        self.assertTrue('tests/mocks/apigateway/resolver/mapping_handlers/basic.py' in file_path)
        self.assertEqual('tests.mocks.apigateway.resolver.mapping_handlers.basic', import_path)

    def test_get_file_and_import_path_module_with_dirty_mapping(self):
        mapping_resolver = MappingModeResolver(base_path=self.base_path, handlers=self.handler_mapping_leading_slashes)
        request = Request(self.basic_request)
        file_path, import_path = mapping_resolver._get_file_and_import_path(request.path)
        self.assertTrue('tests/mocks/apigateway/resolver/mapping_handlers/basic.py' in file_path)
        self.assertEqual('tests.mocks.apigateway.resolver.mapping_handlers.basic', import_path)

    def test_get_file_and_import_path_module_with_disgusting_mapping(self):
        mapping_resolver = MappingModeResolver(base_path=self.base_path, handlers=self.handler_mapping_base_path_leading_slashes)
        request = Request(self.basic_request)
        file_path, import_path = mapping_resolver._get_file_and_import_path(request.path)
        self.assertTrue('tests/mocks/apigateway/resolver/mapping_handlers/basic.py' in file_path)
        self.assertEqual('tests.mocks.apigateway.resolver.mapping_handlers.basic', import_path)

    def test_get_file_and_import_path_module_with_super_disgusting_mapping(self):
        mapping_resolver = MappingModeResolver(base_path=self.base_path, handlers=self.handler_mapping_base_path_mismatch_slashes)
        request = Request(self.basic_request)
        file_path, import_path = mapping_resolver._get_file_and_import_path(request.path)
        self.assertTrue('tests/mocks/apigateway/resolver/mapping_handlers/basic.py' in file_path)
        self.assertEqual('tests.mocks.apigateway.resolver.mapping_handlers.basic', import_path)

    def test_get_endpoint_module(self):
        request = Request(self.basic_request)
        response = Response()
        endpoint_module = self.mapping_resolver.get_endpoint_module(request)
        self.assertTrue(hasattr(endpoint_module, 'post'))
        endpoint_returns = endpoint_module.post(request, response)
        self.assertEqual({'mapping_basic': True}, endpoint_returns.raw)

    def test_get_file_and_import_path_module_nested_route(self):
        request = Request(self.nested_request)
        response = Response()
        endpoint_module = self.mapping_resolver.get_endpoint_module(request)
        self.assertTrue(hasattr(endpoint_module, 'post'))
        endpoint_returns = endpoint_module.post(request, response)
        self.assertDictEqual({'mapping_nested_2_basic': True}, endpoint_returns.raw)

    def test_get_file_and_import_path_module_home_route(self):
        request = Request(self.init_request)
        response = Response()
        endpoint_module = self.mapping_resolver.get_endpoint_module(request)
        self.assertTrue(hasattr(endpoint_module, 'post'))
        endpoint_returns = endpoint_module.post(request, response)
        self.assertDictEqual({'mapping_home_init': True}, endpoint_returns.raw)

    def test_get_file_and_import_path_module_dynamic_route(self):
        request = Request(self.dynamic_request)
        response = Response()
        endpoint_module = self.mapping_resolver.get_endpoint_module(request)
        self.assertTrue(hasattr(endpoint_module, 'post'))
        endpoint_returns = endpoint_module.post(request, response)
        self.assertDictEqual({'mapping_dynamic': True}, endpoint_returns.raw)

    def test_get_file_and_import_path_module_bad_route(self):
        request = Request(self.bad_route_request)
        try:
            self.mapping_resolver.get_endpoint_module(request)
            self.assertTrue(False)
        except ApiException as resolver_error:
            self.assertTrue(isinstance(resolver_error, ApiException))
            self.assertEqual(resolver_error.code, 404)
            self.assertEqual(resolver_error.message, 'route not found')

    def test_get_file_and_import_path_module_empty_route(self):
        variable_request = mock_request.get_dynamic_event(path='unit-test/v1/', method='post')
        request = Request(variable_request)
        response = Response()
        endpoint_module = self.mapping_resolver.get_endpoint_module(request)
        self.assertTrue(hasattr(endpoint_module, 'post'))
        endpoint_returns = endpoint_module.post(request, response)
        self.assertEqual({'mapping_basic': True}, endpoint_returns.raw)



def get_basic():
    return {
        'Records': [
            {
                'messageId': '059f36b4-87a3-44ab-83d2-661975830a7d',
                'receiptHandle': 'AQEBwJnKyrHigUMZj6rYigCgxlaS3SLy0a...',
                'body': '{"lang": "en-us", "sms": true, "email": true, "push": true}',
                'attributes': {
                    'ApproximateReceiveCount': '1',
                    'SentTimestamp': '1545082649183',
                    'SenderId': 'AIDAIENQZJOLO23YVJ4VO',
                    'ApproximateFirstReceiveTimestamp': '1545082649185'
                },
                'messageAttributes': {
                    'SomeString': {
                        'DataType': 'string',
                        'StringValue': 'Some String'
                    },
                    'SomeBinary': {
                        'DataType': 'binary',
                        'BinaryValue': 'Some Binary'
                    }
                },
                'md5OfBody': 'e4e68fb7bd0e697a0ae8f1bb342846b3',
                'eventSource': 'aws:sqs',
                'eventSourceARN': 'arn:aws:sqs:us-east-2:123456789012:my-queue',
                'awsRegion': 'us-east-2'
            }
        ]
    }



import base64
import json


def get_basic(http_method='POST'):
    return {
        'requestContext': {
            'elb': {
                'targetGroupArn': 'arn:aws:elasticloadbalancing:us-east-1:123456789012:targetgroup/my-target-group/50dc6c495c0c9188'
            }
        },
        'httpMethod': http_method,
        'path': '/v1/kyc/callback',
        'queryStringParameters': {
            'status': 'approved',
            'ref': 'abc-123'
        },
        'headers': {
            'content-type': 'application/json',
            'host': '10.84.140.68',
            'x-forwarded-for': '10.104.8.48',
            'x-forwarded-port': '80',
            'x-forwarded-proto': 'http'
        },
        'body': '{"business_id": "biz-e2e-001", "decision": "APPROVED", "completed": "2026-03-30T12:40:40.353"}',
        'isBase64Encoded': False
    }


def get_base64():
    body = json.dumps({'business_id': 'biz-e2e-002', 'decision': 'DENIED'})
    return {
        'requestContext': {
            'elb': {
                'targetGroupArn': 'arn:aws:elasticloadbalancing:us-east-1:123456789012:targetgroup/my-target-group/50dc6c495c0c9188'
            }
        },
        'httpMethod': 'POST',
        'path': '/v1/kyc/callback',
        'queryStringParameters': {},
        'headers': {
            'content-type': 'application/json',
            'host': '10.84.140.68',
            'x-forwarded-for': '10.104.8.48',
            'x-forwarded-port': '80',
            'x-forwarded-proto': 'http'
        },
        'body': base64.b64encode(body.encode('utf-8')).decode('utf-8'),
        'isBase64Encoded': True
    }



import base64

from acai_aws.common.json_helper import JsonHelper
from acai_aws.base.record import BaseRecord


class Record(BaseRecord):

    def __init__(self, record):
        super().__init__(record)
        self._http_operations = {
            'POST': self.CREATED,
            'PUT': self.UPDATED,
            'PATCH': self.UPDATED,
            'DELETE': self.DELETED
        }

    @property
    def body(self):
        raw_body = self._record.get('body', '')
        if self._record.get('isBase64Encoded') and raw_body:
            raw_body = base64.b64decode(raw_body).decode('utf-8')
        return JsonHelper.decode(raw_body)

    @property
    def operation(self):
        return self._http_operations.get(self.http_method, self.UNKNOWN)

    @property
    def http_method(self):
        return self._record.get('httpMethod')

    @property
    def path(self):
        return self._record.get('path')

    @property
    def headers(self):
        return self._record.get('headers', {})

    @property
    def query_params(self):
        return self._record.get('queryStringParameters', {})

    @property
    def source_ip(self):
        return self.headers.get('x-forwarded-for')

    @property
    def target_group_arn(self):
        return self._record.get('requestContext', {}).get('elb', {}).get('targetGroupArn')

    def __str__(self):
        return str({
            'body': self.body,
            'operation': self.operation,
            'http_method': self.http_method,
            'path': self.path,
            'headers': self.headers,
            'query_params': self.query_params,
            'source_ip': self.source_ip,
            'target_group_arn': self.target_group_arn
        })



def get_basic():
    return {
        'eventSource': 'aws:mq',
        'eventSourceArn': 'arn:aws:mq:us-west-2:111122223333:broker:test:b-9bcfa592-423a-4942-879d-eb284b418fc8',
        'messages': [
            {
                'messageID': 'ID:b-9bcfa592-423a-4942-879d-eb284b418fc8-1.mq.us-west-2.amazonaws.com-37557-1234520418293-4:1:1:1:1',
                'messageType': 'jms/text-message',
                'deliveryMode': 1,
                'replyTo': None,
                'type': None,
                'expiration': '60000',
                'priority': 1,
                'correlationId': 'myJMSCoID',
                'redelivered': False,
                'destination': {
                    'physicalName': 'testQueue'
                },
                'data': 'eyJsYW5nIjogImVuLXVzIiwgInNtcyI6IHRydWUsICJlbWFpbCI6IHRydWUsICJwdXNoIjogdHJ1ZX0=',
                'timestamp': 1598827811958,
                'brokerInTime': 1598827811958,
                'brokerOutTime': 1598827811959,
                'properties': {
                    'index': '1',
                    'doAlarm': 'false',
                    'myCustomProperty': 'value'
                }
            }
        ]
    }



from acai_aws.apigateway.requirements import requirements

@requirements()
def post(request, response):
    response.body = {'router_directory_auto': request.body}
    return response

@requirements()
def get(request, response):
    response.body = {'page_number': 1, 'data': {'id': '2'}}
    return response

@requirements(
    required_response='v1-required-response'
)
def patch(request, response):
    response.body = {'page_number': 1, 'data': {'id': '2'}}
    return response

@requirements()
def put(request, response):
    response.body = {'page_number': 1, 'bad-put': {'id': 2}}
    return response

@requirements(
     required_response='v1-required-response'
)
def delete(request, response):
    response.body = {'page_number': 1, 'bad-delete': {'id': 2}}
    return response



import time

from acai_aws.apigateway.requirements import requirements

@requirements()
def post(request, response):
    time.sleep(5)
    response.body = {'router_directory_basic_timeout': request.body}
    return response

@requirements(timeout=1)
def get(request, response):
    time.sleep(5)
    response.body = {'router_directory_basic_timeout': request.body}
    return response



import copy
import os

from pydantic import BaseModel


class HandlerModule:
    def __init__(self, handler_base, file_path, module, method, base):  # pylint: disable=too-many-positional-arguments
        self.__handler_base = handler_base
        self.__file_path = file_path
        self.__module = module
        self.__method = method
        self.__base_path = base.strip(os.sep)
        self.__func = getattr(module, method)
        self.__requirements = getattr(self.__func, 'requirements', {})
        self.__route_path = ''

    @property
    def file_path(self):
        return self.__file_path

    @property
    def module(self):
        return self.__module

    @property
    def method(self):
        return self.__method.lower()

    @property
    def operation_id(self):
        id_prefix = ''.join(r for r in self.route_path.title() if r.isalnum())
        return f'{self.method.title()}{id_prefix}AcaiGenerated'

    @property
    def route_path(self):
        if self.__requirements.get('required_route'):
            self.__route_path = self.__requirements['required_route']
        if not self.__route_path:
            self.__route_path = self.__compose_route_path()
        return self.__route_path if self.__route_path.startswith('/') else f'/{self.__route_path}'

    @property
    def deprecated(self):
        return bool(self.__requirements.get('deprecated'))

    @property
    def summary(self):
        return self.__requirements.get('summary')

    @property
    def tags(self):
        return [self.__base_path.replace(os.sep, '-')]

    @property
    def requires_auth(self):
        return self.__requirements.get('auth_required')

    @property
    def required_headers(self):
        return self.__requirements.get('required_headers', [])

    @property
    def available_headers(self):
        return self.__requirements.get('available_headers', [])

    @property
    def required_query(self):
        return self.__requirements.get('required_query', [])

    @property
    def available_query(self):
        return self.__requirements.get('available_query', [])

    @property
    def required_path_params(self):
        path_params = []
        for path_part in self.route_path.split('/'):
            if '{' in path_part and '}' in path_part:
                cleaned_part = path_part.replace('{', '').replace('}', '')
                path_params.append(cleaned_part)
        return path_params

    @property
    def request_body_schema_name(self):
        return f'{self.method}{self.route_path.replace("/", "-").replace("_", "-").replace("{", "").replace("}", "")}-request-body'

    @property
    def request_body_schema(self):
        return self.__get_schema_body('required_body')

    @property
    def response_body_schema_name(self):
        return f'{self.method}{self.route_path.replace("/", "-").replace("_", "-").replace("{", "").replace("}", "")}-response-body'

    @property
    def response_body_schema(self):
        return self.__get_schema_body('required_response')

    @property
    def response_codes(self):
        return self.__requirements.get('response_codes', {})

    def __compose_route_path(self):
        dirty_route = self.__file_path.split(self.__handler_base)[1]
        no_py_route = dirty_route.replace('.py', '')
        no_init_route = no_py_route.replace('__init__', '')
        clean_route = [self.__base_path]
        for route in no_init_route.split(os.sep):
            if route.startswith('_'):
                route = ''.join(route.split('_')[1])
                route = f'{{{route}}}'
            if route:
                clean_route.append(route)
        return '/'.join(clean_route)

    def __get_schema_body(self, schema_key):
        schema_body = self.__requirements.get(schema_key)
        if not schema_body or isinstance(schema_body, str):
            return None
        if isinstance(schema_body, dict):
            return schema_body
        if issubclass(schema_body, BaseModel):
            return self.__inline_defs(schema_body.model_json_schema())
        return None

    @staticmethod
    def __inline_defs(schema):
        defs = schema.pop('$defs', None)
        if not defs:
            return schema

        def resolve(obj):
            if isinstance(obj, dict):
                if '$ref' in obj and obj['$ref'].startswith('#/$defs/'):
                    name = obj['$ref'].split('/')[-1]
                    return resolve(copy.deepcopy(defs[name])) if name in defs else obj
                return {k: resolve(v) for k, v in obj.items()}
            if isinstance(obj, list):
                return [resolve(item) for item in obj]
            return obj

        return resolve(schema)



import time

from acai_aws.common.records.requirements import requirements


@requirements(verbose=True)
def mock_func_verbose(event):
    return event

@requirements(timeout=1)
def mock_func_timeout(event):
    time.sleep(5)
    return event



import base64
import json
import unittest

from acai_aws.mq.record import Record

from tests.mocks.mq import mock_event


class MQRecordTest(unittest.TestCase):
    basic_record = mock_event.get_basic()['messages'][0]

    def test_record_accepts_event(self):
        record = Record(self.basic_record)
        self.assertEqual(record.operation, record.UNKNOWN)
        self.assertEqual(record.message_id, self.basic_record['messageID'])
        self.assertEqual(record.message_type, self.basic_record['messageType'])
        self.assertDictEqual(record.data, json.loads(base64.b64decode(self.basic_record['data'])))
        self.assertEqual(record.delivery_mode, self.basic_record['deliveryMode'])
        self.assertEqual(record.reply_to, self.basic_record['replyTo'])
        self.assertEqual(record.expiration, self.basic_record['expiration'])
        self.assertEqual(record.record_type, self.basic_record['type'])
        self.assertEqual(record.priority, self.basic_record['priority'])
        self.assertEqual(record.correlation_id, self.basic_record['correlationId'])
        self.assertEqual(record.redelivered, self.basic_record['redelivered'])
        self.assertEqual(record.destination, self.basic_record['destination'])
        self.assertEqual(record.properties, self.basic_record['properties'])
        self.assertEqual(record.time_stamp, self.basic_record['timestamp'])
        self.assertEqual(record.in_time, self.basic_record['brokerInTime'])
        self.assertEqual(record.out_time, self.basic_record['brokerOutTime'])
        self.assertDictEqual(record.body, json.loads(base64.b64decode(self.basic_record['data'])))

    def test_record_prints(self):
        try:
            record = Record(self.basic_record)
            print(record)
            self.assertTrue(True)
        except Exception as error:
            print(error)
            self.assertTrue(False)



import copy
import os
import yaml


class OpenAPIGenerator:

    def __init__(self, output_location):
        file_location = self.__determine_if_openapi_exists(output_location)
        if file_location is not None:
            self.__doc_dict = self.__read_openapi(file_location)
        else:
            self.__doc_dict = self.__get_default_dict()
        self.__existing_path_methods = {}

    @property
    def doc(self):
        return self.__doc_dict

    def delete_unused_paths(self):
        doc_dict = copy.deepcopy(self.__doc_dict)
        for path in doc_dict['paths']:
            if path not in self.__existing_path_methods:
                del self.__doc_dict['paths'][path]
                continue
            for method in doc_dict['paths'][path]:
                if method not in self.__existing_path_methods[path]:
                    del self.__doc_dict['paths'][path][method]

    def add_path_and_method(self, module):
        route_method = self.__get_route_method(module)
        self.__set_route_method_tags(route_method, module)
        self.__set_operation_id(route_method, module)
        self.__set_deprecated(route_method, module)
        self.__set_security(route_method, module)
        self.__set_parameters(route_method, module)
        self.__set_request_body(route_method, module)
        self.__set_responses(route_method, module)
        self.__set_schemas(module)
        if module.route_path not in self.__existing_path_methods:
            self.__existing_path_methods[module.route_path] = []
        self.__existing_path_methods[module.route_path].append(module.method)

    def __get_route_method(self, module):
        paths = self.__doc_dict['paths']
        if not paths.get(module.route_path):
            paths[module.route_path] = {}
        route = paths[module.route_path]
        if not route.get(module.method):
            route[module.method] = {}
        return route[module.method]

    def __set_route_method_tags(self, route_method, module):
        if not route_method.get('tags'):
            route_method['tags'] = []
        route_method['tags'] = route_method['tags'] + module.tags
        route_method['tags'] = list(dict.fromkeys(route_method['tags']))

        route_tags = self.__add_doc_tag(route_method['tags'])
        doc_tags = self.__add_doc_tag(self.__doc_dict['tags'])
        combined_tags = route_tags + doc_tags
        self.__doc_dict['tags'] = [dict(tag) for tag in {tuple(combined_tag.items()) for combined_tag in combined_tags}]

    def __add_doc_tag(self, tags):
        doc_tags = []
        for tag in tags:
            if isinstance(tag, dict):
                doc_tags.append({'name': tag.get('name', 'AcaiGenerated')})
            else:
                doc_tags.append({'name': tag})
        return doc_tags

    def __set_operation_id(self, route_method, module):
        if not route_method.get('operationId'):
            route_method['operationId'] = module.operation_id

    def __set_deprecated(self, route_method, module):
        if not route_method.get('deprecated'):
            route_method['deprecated'] = False
        route_method['deprecated'] = module.deprecated

    def __set_security(self, route_method, module):
        if not route_method.get('security') and module.requires_auth:
            route_method['security'] = [{'AcaiGenerated': []}]

    def __set_parameters(self, route_method, module):
        route_method['parameters'] = []
        self.__set_param(route_method['parameters'], module.required_headers, 'header', True)
        self.__set_param(route_method['parameters'], module.available_headers, 'header', False)
        self.__set_param(route_method['parameters'], module.required_query, 'query', True)
        self.__set_param(route_method['parameters'], module.available_query, 'query', False)
        self.__set_param(route_method['parameters'], module.required_path_params, 'path', True)

    def __set_param(self, parameters, requirements, location, required):
        for requirement in requirements:
            parameters.append({
                'in': location,
                'name': requirement,
                'required': required,
                'schema': {
                    'type': 'string'
                }
            })

    def __set_request_body(self, route_method, module):
        if not route_method.get('requestBody') and module.request_body_schema:
            route_method['requestBody'] = {
                'required': True,
                'content': {
                    'application/json': {
                        'schema': {
                            '$ref': f'#/components/schemas/{module.request_body_schema_name}'
                        }
                    }
                }
            }

    def __set_responses(self, route_method, module):
        if module.response_codes:
            if not route_method.get('responses'):
                route_method['responses'] = {}
            for code, description in module.response_codes.items():
                response_entry = {'description': description}
                if code != 204 and module.response_body_schema:
                    response_entry['content'] = {
                        'application/json': {
                            'schema': {
                                '$ref': f'#/components/schemas/{module.response_body_schema_name}'
                            }
                        }
                    }
                route_method['responses'][str(code)] = response_entry
        elif module.response_body_schema:
            if not route_method.get('responses'):
                route_method['responses'] = {}
            route_method['responses']['200'] = {
                'description': 'OK',
                'content': {
                    'application/json': {
                        'schema': {
                            '$ref': f'#/components/schemas/{module.response_body_schema_name}'
                        }
                    }
                }
            }

    def __set_schemas(self, module):
        if module.request_body_schema:
            self.__doc_dict['components']['schemas'][module.request_body_schema_name] = module.request_body_schema
        if module.response_body_schema:
            self.__doc_dict['components']['schemas'][module.response_body_schema_name] = module.response_body_schema

    def __determine_if_openapi_exists(self, output_locaiton):
        file_locaiton = None
        if os.path.isfile(f'{output_locaiton}/openapi.yml'):
            file_locaiton = f'{output_locaiton}/openapi.yml'
        elif os.path.isfile(f'{output_locaiton}/openapi.json'):
            file_locaiton = f'{output_locaiton}/openapi.json'
        return file_locaiton

    def __read_openapi(self, file_location):
        with open(file_location, encoding='utf-8') as schema_file:
            return yaml.load(schema_file, Loader=yaml.FullLoader)

    def __get_default_dict(self):
        return {
            'openapi': '3.1.0',
            'info': {
                'version': '1.0.0',
                'title': 'Acai Generator',
                'license': {
                    'name': 'MIT',
                    'identifier': 'MIT'
                }
            },
            'tags': [],
            'servers': [],
            'paths': {},
            'components': {
                'securitySchemes': {
                    'AcaiGenerated': {
                        'type': 'apiKey',
                        'in': 'header',
                        'name': 'CHANGE-ME'
                    }
                },
                'schemas': {}
            }
        }



from pydantic import BaseModel

from acai_aws.apigateway.requirements import requirements


class Address(BaseModel):
    street: str
    city: str


class UserWithNestedDefs(BaseModel):
    id: int
    name: str
    address: Address


class SimpleResponse(BaseModel):
    ok: bool


@requirements(
    auth_required=True,
    required_body=UserWithNestedDefs,
    required_response=SimpleResponse,
)
def post(_, response):
    response.body = {'ok': True}
    return response


class NotAPydanticModel:
    pass


@requirements(required_body=NotAPydanticModel)
def put(_, response):
    response.body = {}
    return response



import time

from acai_aws.apigateway.requirements import requirements

call_order = []

class TestDataClass:
    def __init__(self, request):
        self.request = request
        self.initialized = True


def before_call(request, response, reqs):
    before_call.has_been_called = True
    global call_order
    call_order.append('before')
    pass


def after_call(request, response, reqs):
    after_call.has_been_called = True
    global call_order
    call_order.append('after')
    pass


@requirements(
    custom_requiremnent=True,
    before=before_call,
    after=after_call,
    data_class=TestDataClass
)
def post(data_class, response):
    response.body = {'requirements_basic': data_class.initialized}
    return response


@requirements()
def get(_, response):
    time.sleep(5)
    response.body = {'timeout_basic': 'timeout'}
    return response

@requirements(timeout=1)
def patch(_, response):
    time.sleep(5)
    response.body = {'timeout_basic': 'timeout'}
    return response



from acai_aws.apigateway.requirements import requirements

from tests.mocks.common.mock_pydantic_class import UserRequest


@requirements(required_body=UserRequest)
def post(_, response):
    response.body = {'pydantic_pass': True}
    return response


import base64
import json
import unittest

from acai_aws.firehose.record import Record

from tests.mocks.firehose import mock_event


class FirehoseRecordTest(unittest.TestCase):
    basic_record = mock_event.get_basic()['records'][0]

    def test_record_accepts_event(self):
        record = Record(self.basic_record)
        self.assertEqual(record.operation, record.UNKNOWN)
        self.assertDictEqual(record.body, json.loads(base64.b64decode(self.basic_record['data']).decode('utf-8')))
        self.assertEqual(record.record_id, self.basic_record['recordId'])
        self.assertEqual(record.epoc_time_stamp, self.basic_record['approximateArrivalTimestamp'])
        self.assertEqual(record.shard_id, self.basic_record['kinesisRecordMetadata']['shardId'])
        self.assertEqual(record.partition_key, self.basic_record['kinesisRecordMetadata']['partitionKey'])
        self.assertEqual(record.time_stamp, self.basic_record['kinesisRecordMetadata']['approximateArrivalTimestamp'])
        self.assertEqual(record.sequence_number, self.basic_record['kinesisRecordMetadata']['sequenceNumber'])
        self.assertEqual(record.subsequence_number, self.basic_record['kinesisRecordMetadata']['subsequenceNumber'])
        self.assertEqual(record.data, json.loads(base64.b64decode(self.basic_record['data']).decode('utf-8')))

    def test_record_prints(self):
        try:
            record = Record(self.basic_record)
            print(record)
            self.assertTrue(True)
        except Exception as error:
            print(error)
            self.assertTrue(False)



from acai_aws.apigateway.exception import ApiException
from acai_aws.apigateway.requirements import requirements


@requirements()
def post(request, response):
    response.body = {'router_directory_basic': request.body}
    raise ApiException(code=418, key_path='crazy_error', message='I am a teapot')
    return response



def get_basic():
    return {
        'invocationId': 'invoked123',
        'deliveryStreamArn': 'aws:lambda:events',
        'region': 'us-west-2',
        'records': [
            {
                'data': 'eyJsYW5nIjogImVuLXVzIiwgInNtcyI6IHRydWUsICJlbWFpbCI6IHRydWUsICJwdXNoIjogdHJ1ZX0=',
                'recordId': 'record1',
                'approximateArrivalTimestamp': 1510772160000,
                'kinesisRecordMetadata': {
                    'shardId': 'shardId-000000000000',
                    'partitionKey': '4d1ad2b9-24f8-4b9d-a088-76e9947c317a',
                    'approximateArrivalTimestamp': '2012-04-23T18:25:43.511Z',
                    'sequenceNumber': '49546986683135544286507457936321625675700192471156785154',
                    'subsequenceNumber': ''
                }
            }
        ]
    }



import time

from acai_aws.generic.requirements import requirements

from tests.mocks.generic.mock_class import MockDataClass

call_list = []

def before_call(records, reqs):
    before_call.has_been_called = True
    global call_list
    call_list.append('before')


def after_call(records, result, reqs):
    after_call.has_been_called = True
    global call_list
    call_list.append('after')


@requirements(
    custom_requiremnent=True,
    before=before_call,
    after=after_call
)
def mock_generic(event):
    return {'generic': event.body}


@requirements(
    data_class=MockDataClass
)
def mock_generic_dc(data_class):
    return data_class


@requirements(timeout=1)
def mock_timeout(event):
    time.sleep(5)
    return event



import time
import os
from unittest import TestCase, mock

from acai_aws.apigateway.router import Router

from tests.mocks.apigateway import mock_middleware, mock_request


@mock.patch.dict(os.environ, {'LOG_FORMAT': 'INLINE'})
class RouterTimeoutTest(TestCase):
    maxDiff = None
    base_path = 'unit-test/v1'
    handler_path = 'tests/mocks/apigateway/router/directory_handlers'
    schema_path = 'tests/mocks/apigateway/openapi.yml'
    mock_request = mock_request

    def test_global_timeout_works(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            path='unit-test/v1/timeout',
            method='get'
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_path,
            schema=self.schema_path,
            timeout=1
        )
        response = router.route(dynamic_event, None)
        self.assertEqual(408, response['statusCode'])

    def test_local_timeout_works(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            path='unit-test/v1/timeout',
            method='get'
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_path,
            schema=self.schema_path
        )
        response = router.route(dynamic_event, None)
        self.assertEqual(408, response['statusCode'])

    def test_global_timeout_with_middleware_works(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            path='unit-test/v1/timeout',
            method='post'
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_path,
            schema=self.schema_path,
            timeout=1,
            on_timeout=mock_middleware.mock_on_timeout
        )
        response = router.route(dynamic_event, None)
        self.assertEqual(408, response['statusCode'])
        self.assertTrue(mock_middleware.mock_on_timeout.has_been_called)

    def test_local_timeout_with_middleware_works(self):
        dynamic_event = self.mock_request.get_dynamic_event(
            path='unit-test/v1/timeout',
            method='get'
        )
        router = Router(
            base_path=self.base_path,
            handlers=self.handler_path,
            schema=self.schema_path,
            on_timeout=mock_middleware.mock_on_timeout
        )
        response = router.route(dynamic_event, None)
        self.assertEqual(408, response['statusCode'])
        self.assertTrue(mock_middleware.mock_on_timeout.has_been_called)




from acai_aws.dynamodb.requirements import requirements

call_list = []

class MockDataClass:
    def __init__(self, record):
        self.record = record
        self.initialized = True


def before_call(records, reqs):
    before_call.has_been_called = True
    global call_list
    call_list.append('before')


def after_call(records, result, reqs):
    after_call.has_been_called = True
    global call_list
    call_list.append('after')


@requirements(
    custom_requiremnent=True,
    before=before_call,
    after=after_call,
    data_class=MockDataClass,
    operations=['created', 'deleted']
)
def mock_ddb_full(event):
    full_results = []
    for mock_data_class in event.records:
        full_results.append(mock_data_class.initialized)
    return {'ddb_full': full_results}


@requirements()
def mock_ddb_basic(event):
    basic_results = []
    for record in event.records:
        basic_results.append(record.name)
    return {'ddb_basic': basic_results}

@requirements(
    operations=['deleted']
)
def mock_ddb_operation_ignore(event):
    ignored_results = []
    for record in event.records:
        ignored_results.append(record.name)
    return {'ddb_operation_ignore': ignored_results}


@requirements(
    operations=['deleted'],
    raise_operation_error=True
)
def mock_ddb_operation_raise(event):
    ignored_results = []
    for record in event.records:
        ignored_results.append(record.name)
    return {'ddb_operation_ignore': ignored_results}



from typing import List

from pydantic import BaseModel, PositiveInt


class Request(BaseModel):
    test_id: str
    fail_id: str
    object_key: dict[str, str]
    array_number: List[PositiveInt]
    array_objects: List[dict[str, PositiveInt]]


class UserRequest(BaseModel):
    id: PositiveInt
    email: str
    active: bool
    favorites: List[str]
    notification_config: dict[str, bool]


from acai_aws.apigateway.requirements import requirements

@requirements(required_route='/bad/dynamic/{id}')
def post(_, response):
    response.body = {'mapping_dynamic_bad': True}
    return response

@requirements(required_route='/bad_dynamic/{id')
def get(_, response):
    response.body = {'mapping_dynamic_bad': True}
    return response



def get():
    return {
        'openapi': '3.1.0',
        'info': {
            'version': '1.0.0',
            'title': 'Acai Generator',
            'license': {
                'name': 'MIT'
            }
        },
        'tags': [
            {
                'name': 'acai_aws-example'
            }
        ],
        'servers': [],
        'paths': {
            '/acai_aws/example/basic': {
                'get': {
                    'tags': [
                        'acai_aws-example'
                    ],
                    'operationId': 'GetAcaiAwsExampleBasicAcaiGenerated',
                    'deprecated': False,
                    'parameters': [
                        {
                            'in': 'query',
                            'name': 'basic_id',
                            'required': True,
                            'schema': {
                                'type': 'string'
                            }
                        },
                        {
                            'in': 'query',
                            'name': 'first',
                            'required': False,
                            'schema': {
                                'type': 'string'
                            }
                        },
                        {
                            'in': 'query',
                            'name': 'last',
                            'required': False,
                            'schema': {
                                'type': 'string'
                            }
                        }
                    ]
                },
                'post': {
                    'tags': [
                        'acai_aws-example'
                    ],
                    'operationId': 'PostAcaiAwsExampleBasicAcaiGenerated',
                    'deprecated': False,
                    'parameters': [],
                    'requestBody': {
                        'required': True,
                        'content': {
                            'application/json': {
                                'schema': {
                                    '$ref': '#/components/schemas/post-acai-aws-example-basic-request-body'
                                }
                            }
                        }
                    }
                }
            },
            '/nested_1/nested_2/{id}': {
                'delete': {
                    'tags': [
                        'acai_aws-example'
                    ],
                    'operationId': 'DeleteNested1Nested2IdAcaiGenerated',
                    'deprecated': False,
                    'parameters': [
                        {
                            'in': 'path',
                            'name': 'id',
                            'required': True,
                            'schema': {
                                'type': 'string'
                            }
                        }
                    ]
                },
                'patch': {
                    'tags': [
                        'acai_aws-example'
                    ],
                    'operationId': 'PatchNested1Nested2IdAcaiGenerated',
                    'deprecated': False,
                    'parameters': [
                        {
                            'in': 'path',
                            'name': 'id',
                            'required': True,
                            'schema': {
                                'type': 'string'
                            }
                        }
                    ],
                    'requestBody': {
                        'required': True,
                        'content': {
                            'application/json': {
                                'schema': {
                                    '$ref': '#/components/schemas/patch-nested-1-nested-2-id-request-body'
                                }
                            }
                        }
                    }
                }
            },
            '/acai_aws/example/nested_1/nested_2/resource': {
                'put': {
                    'tags': [
                        'acai_aws-example'
                    ],
                    'operationId': 'PutAcaiAwsExampleNested1Nested2ResourceAcaiGenerated',
                    'deprecated': False,
                    'parameters': []
                }
            },
            '/acai_aws/example/{dynamic}': {
                'get': {
                    'tags': [
                        'acai_aws-example'
                    ],
                    'operationId': 'GetAcaiAwsExampleDynamicAcaiGenerated',
                    'deprecated': False,
                    'security': [
                        {
                            'AcaiGenerated': []
                        }
                    ],
                    'parameters': [
                        {
                            'in': 'header',
                            'name': 'x-dynamic-key',
                            'required': True,
                            'schema': {
                                'type': 'string'
                            }
                        },
                        {
                            'in': 'path',
                            'name': 'dynamic',
                            'required': True,
                            'schema': {
                                'type': 'string'
                            }
                        }
                    ],
                    'responses': {
                        '200': {
                            'description': 'OK',
                            'content': {
                                'application/json': {
                                    'schema': {
                                        '$ref': '#/components/schemas/get-acai-aws-example-dynamic-response-body'
                                    }
                                }
                            }
                        }
                    }
                },
                'post': {
                    'tags': [
                        'acai_aws-example'
                    ],
                    'operationId': 'PostAcaiAwsExampleDynamicAcaiGenerated',
                    'deprecated': False,
                    'security': [
                        {
                            'AcaiGenerated': []
                        }
                    ],
                    'parameters': [
                        {
                            'in': 'path',
                            'name': 'dynamic',
                            'required': True,
                            'schema': {
                                'type': 'string'
                            }
                        }
                    ],
                    'requestBody': {
                        'required': True,
                        'content': {
                            'application/json': {
                                'schema': {
                                    '$ref': '#/components/schemas/post-acai-aws-example-dynamic-request-body'
                                }
                            }
                        }
                    }
                }
            }
        },
        'components': {
            'securitySchemes': {
                'AcaiGenerated': {
                    'type': 'apiKey',
                    'in': 'header',
                    'name': 'CHANGE-ME'
                }
            },
            'schemas': {
                'post-acai-aws-example-basic-request-body': {
                    'properties': {
                        'id': {
                            'exclusiveMinimum': 0,
                            'title': 'Id',
                            'type': 'integer'
                        },
                        'email': {
                            'title': 'Email',
                            'type': 'string'
                        },
                        'active': {
                            'title': 'Active',
                            'type': 'boolean'
                        },
                        'favorites': {
                            'items': {
                                'type': 'string'
                            },
                            'title': 'Favorites',
                            'type': 'array'
                        },
                        'notification_config': {
                            'additionalProperties': {
                                'type': 'boolean'
                            },
                            'title': 'Notification Config',
                            'type': 'object'
                        }
                    },
                    'required': [
                        'id',
                        'email',
                        'active',
                        'favorites',
                        'notification_config'
                    ],
                    'title': 'UserRequest',
                    'type': 'object'
                },
                'patch-nested-1-nested-2-id-request-body': {
                    'type': 'object',
                    'required': [],
                    'additionalProperties': False,
                    'properties': {
                        'grower_id': {
                            'type': 'string'
                        },
                        'body': {
                            'type': 'object'
                        },
                        'dict': {
                            'type': 'boolean'
                        }
                    }
                },
                'get-acai-aws-example-dynamic-response-body': {
                    'type': 'object',
                    'required': [
                        'dynamic_id',
                        'dynamic_bool',
                        'dynamic_message'
                    ],
                    'additionalProperties': False,
                    'properties': {
                        'dynamic_id': {
                            'type': 'int'
                        },
                        'dynamic_bool': {
                            'type': 'boolean'
                        },
                        'dynamic_message': {
                            'type': 'string'
                        }
                    }
                },
                'post-acai-aws-example-dynamic-request-body': {
                    'properties': {
                        'id': {
                            'exclusiveMinimum': 0,
                            'title': 'Id',
                            'type': 'integer'
                        },
                        'email': {
                            'title': 'Email',
                            'type': 'string'
                        },
                        'active': {
                            'title': 'Active',
                            'type': 'boolean'
                        },
                        'favorites': {
                            'items': {
                                'type': 'string'
                            },
                            'title': 'Favorites',
                            'type': 'array'
                        },
                        'notification_config': {
                            'additionalProperties': {
                                'type': 'boolean'
                            },
                            'title': 'Notification Config',
                            'type': 'object'
                        }
                    },
                    'required': [
                        'id',
                        'email',
                        'active',
                        'favorites',
                        'notification_config'
                    ],
                    'title': 'UserRequest',
                    'type': 'object'
                }
            }
        }
    }


from acai_aws.apigateway.requirements import requirements

from tests.mocks.common.mock_pydantic_class import UserRequest
# required_route='/user/{user_id}/item'
#


@requirements(
    available_query=['first', 'last'],
    required_query=['basic_id']
)
def get(_, response):
    response.body = {'basic-openapigenerator': True}
    return response


@requirements(
    required_body=UserRequest
)
def post(_, response):
    response.body = {'basic-openapigenerator': True}
    return response



from acai_aws.apigateway.requirements import requirements


@requirements(
    required_body='some-schema'
)
def post(_, response):
    response.body = {'endpoint_directory_basic': 'post'}
    return response


def patch(_, response):
    response.body = {'endpoint_directory_basic': 'patch'}
    return response


@requirements(
    auth_required=True
)
def get(_, response):
    response.body = {'endpoint_directory_basic': 'get'}
    return response


@requirements(
    required_route='/some/route/{id}'
)
def delete(_, response):
    response.body = {'endpoint_directory_basic': 'delete'}
    return response


@requirements(
    custom_list=[1, 2, 3],
    custom_dict={'key': 'value'},
    custom_simple=1
)
def put(_, response):
    response.body = {'endpoint_directory_basic': 'put'}
    return response


@requirements(
    required_response='somme-response-schema'
)
def search(_, response):
    response.body = {'endpoint_directory_basic': 'search'}
    return response



import unittest

from acai_aws.apigateway.request import Request
from acai_aws.apigateway.response import Response
from acai_aws.common.validator import Validator

from tests.mocks.apigateway import mock_request
from tests.mocks.common.mock_pydantic_class import UserRequest


class ValidatorTest(unittest.TestCase):
    schema_path = 'tests/mocks/common/openapi.yml'

    def setUp(self):
        self.validator = Validator(schema=self.schema_path)

    def test_empty_validation(self):
        request = Request(mock_request.get_basic_for_validation())
        response = Response()
        requirements = {}
        self.validator.validate_request(request, response, requirements)
        self.assertFalse(response.has_errors)

    def test_required_headers_pass(self):
        request = Request(mock_request.get_basic_for_validation())
        response = Response()
        requirements = {
            'required_headers': ['content-type']
        }
        self.validator.validate_request(request, response, requirements)
        self.assertFalse(response.has_errors)

    def test_required_headers_fails(self):
        request = Request(mock_request.get_basic_for_validation())
        response = Response()
        requirements = {
            'required_headers': ['content-type-fails']
        }
        self.validator.validate_request(request, response, requirements)
        self.assertTrue(response.has_errors)
        self.assertEqual('{"errors": [{"key_path": "headers", "message": "Please provide content-type-fails in headers"}]}', response.full['body'])

    def test_available_headers_pass(self):
        request = Request(mock_request.get_basic_for_validation())
        response = Response()
        requirements = {
            'available_headers': ['content-type']
        }
        self.validator.validate_request(request, response, requirements)
        self.assertFalse(response.has_errors)

    def test_available_headers_fails(self):
        request = Request(mock_request.get_basic_for_validation())
        response = Response()
        requirements = {
            'available_headers': ['content-type-fail']
        }
        self.validator.validate_request(request, response, requirements)
        self.assertTrue(response.has_errors)
        self.assertEqual('{"errors": [{"key_path": "headers", "message": "content-type is not an available headers"}]}', response.full['body'])

    def test_required_query_pass(self):
        request = Request(mock_request.get_basic_for_validation())
        response = Response()
        requirements = {
            'required_query': ['email']
        }
        self.validator.validate_request(request, response, requirements)
        self.assertFalse(response.has_errors)

    def test_required_query_fail(self):
        request = Request(mock_request.get_basic_for_validation())
        response = Response()
        requirements = {
            'required_query': ['email', 'other']
        }
        self.validator.validate_request(request, response, requirements)
        self.assertTrue(response.has_errors)
        self.assertEqual('{"errors": [{"key_path": "query_params", "message": "Please provide other in query_params"}]}', response.full['body'])

    def test_available_query_pass(self):
        request = Request(mock_request.get_basic_for_validation())
        response = Response()
        requirements = {
            'available_query': ['email', 'other']
        }
        self.validator.validate_request(request, response, requirements)
        self.assertFalse(response.has_errors)

    def test_available_query_fail(self):
        request = Request(mock_request.get_basic_for_validation())
        response = Response()
        requirements = {
            'available_query': ['other']
        }
        self.validator.validate_request(request, response, requirements)
        self.assertTrue(response.has_errors)
        self.assertEqual('{"errors": [{"key_path": "query_params", "message": "email is not an available query_params"}]}', response.full['body'])

    def test_required_body_pass(self):
        request = Request(mock_request.get_basic_passing_for_required_body_validation())
        response = Response()
        requirements = {
            'required_body': 'v1-required-body-test'
        }
        self.validator.validate_request(request, response, requirements)
        self.assertFalse(response.has_errors)

    def test_required_body_fail(self):
        request = Request(mock_request.get_basic_failing_for_required_body_validation())
        response = Response()
        requirements = {
            'required_body': 'v1-required-body-test'
        }
        self.validator.validate_request(request, response, requirements)
        self.assertTrue(response.has_errors)
        self.assertEqual('{"errors": [{"key_path": "root", "message": "\'id\' is a required property"}]}', response.body)

    def test_validate_request_with_openapi_passes(self):
        request = Request(mock_request.get_auto_validated_data())
        response = Response()
        self.validator.validate_request_with_openapi(request, response)

    def test_required_pydantic_body_pass(self):
        request = Request(mock_request.get_basic_passing_for_required_body_validation())
        response = Response()
        requirements = {
            'required_body': UserRequest
        }
        self.validator.validate_request(request, response, requirements)
        self.assertFalse(response.has_errors)

    def test_required_pydantic_body_fail(self):
        request = Request(mock_request.get_basic_failing_for_required_body_validation())
        response = Response()
        requirements = {
            'required_body': UserRequest
        }
        self.validator.validate_request(request, response, requirements)
        self.assertTrue(response.has_errors)
        self.assertEqual('{"errors": [{"key_path": "id", "message": "Field required"}]}', response.body)



from acai_aws.apigateway.requirements import requirements

@requirements(
    auth_required=True
)
def post(request, response):
    response.body = {'router_pattern_basic': request.body}
    return response



import unittest
import logging

from acai_aws.apigateway.resolver.cache import ResolverCache
from acai_aws.apigateway.endpoint import Endpoint
from acai_aws.apigateway.resolver.importer import ResolverImporter


class ResolverCacheTest(unittest.TestCase):
    handler_path = 'tests/mocks/apigateway/cacher/directory_handlers'

    def setUp(self):
        self.importer = ResolverImporter(handlers=self.handler_path, mode='directory')

    def test_cache_settings(self):
        cacher = ResolverCache()
        self.assertTrue(cacher.CACHE_ALL == 'all')
        self.assertTrue(cacher.CACHE_STATIC == 'static-only')
        self.assertTrue(cacher.CACHE_DYNAMIC == 'dynamic-only')

    def test_cache_put(self):
        file_path = f'{self.handler_path}/basic.py'
        import_path = 'tests.mocks.apigateway.cacher.directory_handlers.basic'
        endpoint_module = self.importer.import_module_from_file(file_path, import_path)
        endpoint = Endpoint(endpoint_module, 'post')
        cacher = ResolverCache()
        try:
            cacher.put('post::/unit-test/v1/cacher/basic', endpoint, False, {})
            self.assertTrue(True)
        except Exception as exception:
            logging.exception(exception)
            self.assertTrue(False)

    def test_cache_put_and_get(self):
        file_path = f'{self.handler_path}/basic.py'
        import_path = 'tests.mocks.apigateway.cacher.directory_handlers.basic'
        endpoint_module = self.importer.import_module_from_file(file_path, import_path)
        endpoint = Endpoint(endpoint_module, 'patch')
        cacher = ResolverCache()
        cacher.put('patch::/unit-test/v1/cacher/basic', endpoint, False, {})
        cached = cacher.get('patch::/unit-test/v1/cacher/basic')
        self.assertTrue(cached['endpoint'].has_requirements)

    def test_cache_get_miss(self):
        cacher = ResolverCache()
        cached = cacher.get('get::/unit-test/v1/cacher/basic')
        self.assertTrue(cached.get('endpoint') is None)

    def test_cache_lru_sequence_size_reaches_capacity_and_result_in_miss(self):
        file_path = f'{self.handler_path}/basic.py'
        import_path = 'tests.mocks.apigateway.cacher.directory_handlers.basic'
        endpoint_module = self.importer.import_module_from_file(file_path, import_path)
        post_endpoint = Endpoint(endpoint_module, 'post')
        patch_endpoint = Endpoint(endpoint_module, 'patch')
        get_endpoint = Endpoint(endpoint_module, 'get')
        cacher = ResolverCache(cache_size=1)
        cacher.put('get::/unit-test/v1/cacher/basic', get_endpoint, False, {})
        get_cache_result = cacher.get('get::/unit-test/v1/cacher/basic')
        self.assertFalse(get_cache_result['endpoint'].has_requirements)
        cacher.put('patch::/unit-test/v1/cacher/basic', patch_endpoint, False, {})
        cacher.get('patch::/unit-test/v1/cacher/basic')
        cacher.put('post::/unit-test/v1/cacher/basic', post_endpoint, False, {})
        cacher.get('post::/unit-test/v1/cacher/basic')
        miss_cache_result = cacher.get('get::/unit-test/v1/cacher/basic')
        self.assertTrue(miss_cache_result.get('endpoint') is None)

    def test_cache_size_none_always_miss(self):
        file_path = f'{self.handler_path}/basic.py'
        import_path = 'tests.mocks.apigateway.cacher.directory_handlers.basic'
        endpoint_module = self.importer.import_module_from_file(file_path, import_path)
        get_endpoint = Endpoint(endpoint_module, 'get')
        cacher = ResolverCache(cache_size=None)
        cacher.put('get::/unit-test/v1/cacher/basic', get_endpoint, False, {})
        get_cache_result = cacher.get('get::/unit-test/v1/cacher/basic')
        self.assertTrue(get_cache_result.get('endpoint') is None)

    def test_cache_static_only(self):
        file_path = f'{self.handler_path}/basic.py'
        import_path = 'tests.mocks.apigateway.cacher.directory_handlers.basic'
        endpoint_module = self.importer.import_module_from_file(file_path, import_path)
        get_endpoint = Endpoint(endpoint_module, 'get')
        cacher = ResolverCache(cache_mode='static-only')
        cacher.put('get::/unit-test/v1/cacher/basic', get_endpoint, False, {})
        get_cache_result = cacher.get('get::/unit-test/v1/cacher/basic')
        self.assertFalse(get_cache_result['endpoint'].has_requirements)
        patch_endpoint = Endpoint(endpoint_module, 'patch')
        cacher.put('patch::/unit-test/v1/cacher/basic', patch_endpoint, True, {'1': 1})
        patch_cache_result = cacher.get('patch::/unit-test/v1/cacher/basic')
        self.assertTrue(patch_cache_result.get('endpoint') is None)

    def test_cache_dynamic_only(self):
        file_path = f'{self.handler_path}/basic.py'
        import_path = 'tests.mocks.apigateway.cacher.directory_handlers.basic'
        endpoint_module = self.importer.import_module_from_file(file_path, import_path)
        get_endpoint = Endpoint(endpoint_module, 'get')
        cacher = ResolverCache(cache_mode='dynamic-only')
        patch_endpoint = Endpoint(endpoint_module, 'patch')
        cacher.put('patch::/unit-test/v1/cacher/basic', patch_endpoint, True, {'1': 1})
        patch_cache_result = cacher.get('patch::/unit-test/v1/cacher/basic')
        self.assertTrue(patch_cache_result['endpoint'].has_requirements)
        cacher.put('get::/unit-test/v1/cacher/basic', get_endpoint, False, {})
        get_cache_result = cacher.get('get::/unit-test/v1/cacher/basic')
        self.assertTrue(get_cache_result.get('endpoint') is None)

    def test_cache_remembers_is_dynamic_route(self):
        file_path = f'{self.handler_path}/basic.py'
        import_path = 'tests.mocks.apigateway.cacher.directory_handlers.basic'
        endpoint_module = self.importer.import_module_from_file(file_path, import_path)
        get_endpoint = Endpoint(endpoint_module, 'get')
        cacher = ResolverCache()
        cacher.put('get::/unit-test/v1/cacher/basic', get_endpoint, True, {'1': 1})
        cached = cacher.get('get::/unit-test/v1/cacher/basic')
        self.assertTrue(cached['is_dynamic_route'])

    def test_cache_remembers_dynamic_parts(self):
        dynamic_parts = {'1': 1}
        file_path = f'{self.handler_path}/basic.py'
        import_path = 'tests.mocks.apigateway.cacher.directory_handlers.basic'
        endpoint_module = self.importer.import_module_from_file(file_path, import_path)
        get_endpoint = Endpoint(endpoint_module, 'get')
        cacher = ResolverCache()
        cacher.put('get::/unit-test/v1/cacher/basic', get_endpoint, True, dynamic_parts)
        cached = cacher.get('get::/unit-test/v1/cacher/basic')
        self.assertDictEqual(cached['dynamic_parts'], dynamic_parts)



def mock_before_all(request, response, requirements):
    mock_before_all.has_been_called = True


def mock_after_all(request, response, requirements):
    mock_after_all.has_been_called = True


def mock_with_auth(request, response, requirements):
    mock_with_auth.has_been_called = True


def mock_on_error(request, response, error):
    mock_on_error.has_been_called = True


def mock_on_error_exception(request, response, error):
    mock_on_error_exception.has_been_called = True
    raise Exception('something went wrong with middleware')


def mock_on_timeout(request, response, error):
    mock_on_timeout.has_been_called = True



def get_basic(operation='insert'):
    return {
        'eventSourceArn': 'arn:aws:rds:us-east-1:123456789012:cluster:canaryclusterb2a659a2-qo5tcmqkcl03',
        'events': [
            {
                'event': {
                    '_id': {
                        '_data': '0163eeb6e7000000090100000009000041e1'
                    },
                    'clusterTime': {
                        '$timestamp': {
                            't': 1676588775,
                            'i': 9
                        }
                    },
                    'documentKey': {
                        '_id': {
                            '$oid': '63eeb6e7d418cd98afb1c1d7'
                        }
                    },
                    'fullDocument': {
                        '_id': {
                            '$oid': '63eeb6e7d418cd98afb1c1d7'
                        },
                        'lang': 'en-us',
                        'sms': True,
                        'email': True,
                        'push': True
                    },
                    'ns': {
                        'db': 'test_database',
                        'coll': 'test_collection'
                    },
                    'operationType': operation
                }
            }
        ],
        'eventSource': 'aws:docdb'
    }



import logging

from acai_aws.apigateway.exception import ApiException, ApiTimeOutException
from acai_aws.apigateway.request import Request
from acai_aws.apigateway.resolver import Resolver
from acai_aws.apigateway.response import Response
from acai_aws.apigateway.config_validator import ConfigValidator
from acai_aws.common.validator import Validator
from acai_aws.common import logger


class Router:
    request_class = Request
    response_class = Response

    def __init__(self, **kwargs):
        ConfigValidator.validate(**kwargs)
        self.__before_all = kwargs.get('before_all')
        self.__after_all = kwargs.get('after_all')
        self.__with_auth = kwargs.get('with_auth')
        self.__on_error = kwargs.get('on_error')
        self.__on_timeout = kwargs.get('on_timeout')
        self.__cors = kwargs.get('cors', True)
        self.__timeout = kwargs.get('timeout', None)
        self.__output_error = kwargs.get('output_error', False)
        self.__verbose = kwargs.get('verbose_logging', False)
        self.__auto_validate = kwargs.get('auto_validate', False)
        self.__validate_response = kwargs.get('validate_response', False)
        self.__resolver = Resolver(**kwargs)
        self.__validator = Validator(**kwargs)

    def auto_load(self):
        self.__resolver.auto_load()
        self.__validator.auto_load()

    def route(self, event, context):
        request = self.request_class(event, context, self.__timeout)
        response = self.response_class(cors=self.__cors)
        try:
            self.__log_verbose(title='request-received', log={'request': request})
            self.__run_route_procedure(request, response)
        except ApiTimeOutException as timeout_error:
            kwargs = {'code': timeout_error.code, 'key_path': timeout_error.key_path, 'message': timeout_error.message, 'error': timeout_error}
            self.__handle_error(request, response, self.__on_timeout, **kwargs)
        except ApiException as api_error:
            kwargs = {'code': api_error.code, 'key_path': api_error.key_path, 'message': api_error.message, 'error': api_error}
            self.__handle_error(request, response, self.__on_error, **kwargs)
        except Exception as error:
            output = str(error) if self.__output_error else 'internal service error'
            kwargs = {'code': 500, 'key_path': 'unknown', 'message': output, 'error': error}
            self.__handle_error(request, response, **kwargs)
        self.__log_verbose(title='request-processed', log={'request': request, 'response': response})
        return response.full

    def __run_route_procedure(self, request, response):
        endpoint = self.__resolver.get_endpoint(request)
        self.__run_before_all(request, response, endpoint)
        self.__run_with_auth(request, response, endpoint)
        self.__run_request_validation(request, response, endpoint)
        if not response.has_errors:
            endpoint.run(request, response)
        self.__run_response_validation(request, response, endpoint)
        self.__run_after_all(request, response, endpoint)
        return response

    def __run_before_all(self, request, response, endpoint):
        if not response.has_errors and self.__before_all and callable(self.__before_all):
            self.__before_all(request, response, endpoint.requirements)

    def __run_with_auth(self, request, response, endpoint):
        if not response.has_errors and self.__with_auth and callable(self.__with_auth):
            if (self.__auto_validate and self.__validator.request_has_security(request)) or endpoint.requires_auth:
                self.__with_auth(request, response, endpoint.requirements)

    def __run_request_validation(self, request, response, endpoint):
        if not response.has_errors and self.__auto_validate:
            self.__validator.validate_request_with_openapi(request, response)
        elif not response.has_errors and endpoint.has_requirements:
            self.__validator.validate_request(request, response, endpoint.requirements)

    def __run_response_validation(self, request, response, endpoint):
        if not response.has_errors and self.__auto_validate and self.__validate_response:
            self.__validator.validate_response_with_openapi(request, response)
        elif not response.has_errors and self.__validate_response and endpoint.has_required_response:
            self.__validator.validate_response(response, endpoint.requirements)

    def __run_after_all(self, request, response, endpoint):
        if not response.has_errors and self.__after_all and callable(self.__after_all):
            self.__after_all(request, response, endpoint.requirements)

    def __handle_error(self, request, response, error_func=None, **kwargs):
        try:
            response.code = kwargs['code']
            response.set_error(key_path=kwargs['key_path'], message=kwargs['message'])
            if error_func and callable(error_func):
                error_func(request, response, kwargs.get('error'))
            else:
                logger.log(level='ERROR', log={'request': request, 'response': response, 'error': kwargs})
        except Exception as exception:
            logging.exception(exception)

    def __log_verbose(self, title, log):
        if self.__verbose:
            logger.log(level='INFO', log={'title': title, 'log': log})



def get_basic():
    return {
        'Records': [
            {
                'kinesis': {
                    'kinesisSchemaVersion': '1.0',
                    'partitionKey': '1',
                    'sequenceNumber': '49590338271490256608559692538361571095921575989136588898',
                    'data': 'eyJsYW5nIjogImVuLXVzIiwgInNtcyI6IHRydWUsICJlbWFpbCI6IHRydWUsICJwdXNoIjogdHJ1ZX0=',
                    'approximateArrivalTimestamp': 1545084650.987
                },
                'eventSource': 'aws:kinesis',
                'eventVersion': '1.0',
                'eventID': 'shardId-000000000006:49590338271490256608559692538361571095921575989136588898',
                'eventName': 'aws:kinesis:record',
                'invokeIdentityArn': 'arn:aws:iam::123456789012:role/lambda-role',
                'awsRegion': 'us-east-2',
                'eventSourceARN': 'arn:aws:kinesis:us-east-2:123456789012:stream/lambda-stream'
            }
        ]
    }



from acai_aws.apigateway.requirements import requirements

from tests.mocks.common.mock_pydantic_class import UserRequest


@requirements(
    auth_required=True,
    required_body=UserRequest
)
def post(_, response):
    response.body = {'dynamic': True}
    return response


@requirements(
    auth_required=True,
    required_headers=['x-dynamic-key'],
    required_response={
        'type': 'object',
        'required': ['dynamic_id', 'dynamic_bool', 'dynamic_message'],
        'additionalProperties': False,
        'properties': {
            'dynamic_id': {'type': 'int'},
            'dynamic_bool': {'type': 'boolean'},
            'dynamic_message': {'type': 'string'}
        },
    }
)
def get(_, response):
    response.body = {'dynamic': True}
    return response



import datetime
import os
import traceback

import jsonpickle


class CommonLogger:

    def __init__(self):
        self.__json = jsonpickle
        env_format = os.getenv('LOG_FORMAT', 'JSON') or 'JSON'
        self.__format = env_format.strip().upper()
        self.__log_level = os.getenv('LOG_LEVEL', 'INFO')
        self.__json.set_encoder_options('simplejson', use_decimal=True)
        self.__json.set_preferred_backend('simplejson')
        self.log_levels = {
            'DEBUG': 0,
            'INFO': 1,
            'WARN': 2,
            'WARNING': 2,
            'ERROR': 3,
            'CRITICAL': 4,
            'FATAL': 4
        }
        if self.__format not in ['JSON', 'PRETTY', 'INLINE']:
            raise ValueError(f'LOG_FORMAT ENV must be either `JSON`, `PRETTY`, or `INLINE`, recieved: {self.__format}')

    def log(self, **kwargs):
        default_log = {'level': kwargs.get('level', 'INFO'), 'log': kwargs.get('log', {})}
        if self.__should_log(default_log['level']) and self.__format == 'JSON':
            self.__log_json(**kwargs)
        elif self.__should_log(default_log['level']) and self.__format == 'PRETTY':
            self.__log_json(pretty=True, **kwargs)
        elif self.__should_log(default_log['level']) and self.__format == 'INLINE':
            self.__log_inline(**kwargs)

    def __get_traceback(self):
        trace = traceback.format_exc()
        if str(trace) != 'NoneType: None\n':
            return trace
        return ''

    def __should_log(self, level):
        current_log_level = self.log_levels[level]
        log_level_setting = self.log_levels[self.__log_level]
        return current_log_level >= log_level_setting

    def __log_json(self, pretty=False, **kwargs):
        print(self.__json.encode({
            'level': kwargs['level'],
            'time': datetime.datetime.now(datetime.timezone.utc).isoformat(),
            'trace': [trace.strip() for trace in self.__get_traceback().split('\n') if trace],
            'log': kwargs['log']
        }, indent=4 if pretty else None))

    def __log_inline(self, **kwargs):
        timestamp = datetime.datetime.now(datetime.timezone.utc).isoformat()
        trace = self.__get_traceback().strip().replace('\n', ' | ')
        log_value = kwargs['log']
        if not isinstance(log_value, str):
            log_value = str(log_value)
        inline_message = f"{kwargs['level']}|time={timestamp} log={log_value}"
        if trace:
            inline_message = f"{inline_message} trace={trace}"
        print(inline_message)



import unittest

from acai_aws.alb.response import Response


class ALBResponseTest(unittest.TestCase):
    maxDiff = None

    def test_response_includes_status_description_on_full(self):
        response = Response()
        response.body = {'ok': True}
        full = response.full
        self.assertEqual('200 OK', full['statusDescription'])
        self.assertEqual(200, full['statusCode'])

    def test_response_status_description_for_known_codes(self):
        response = Response()
        response.code = 404
        response.body = {'errors': [{'key_path': 'route', 'message': 'not found'}]}
        self.assertEqual('404 Not Found', response.full['statusDescription'])

    def test_response_status_description_unknown_code_falls_back_to_ok(self):
        response = Response()
        response.code = 599
        response.body = {'msg': 'weird'}
        self.assertEqual('599 OK', response.full['statusDescription'])

    def test_response_full_includes_alb_required_keys(self):
        response = Response()
        response.body = {'ok': True}
        full = response.full
        self.assertIn('statusCode', full)
        self.assertIn('statusDescription', full)
        self.assertIn('body', full)
        self.assertIn('headers', full)
        self.assertIn('isBase64Encoded', full)


if __name__ == '__main__':
    unittest.main()



import unittest

from pydantic import BaseModel

from acai_aws.apigateway.openapi.generator import OpenAPIGenerator
from acai_aws.apigateway.openapi.handler.importer import HandlerImporter
from acai_aws.apigateway.openapi.handler.module import HandlerModule
from acai_aws.apigateway.request import Request
from acai_aws.common.schema import Schema


class SchemaGapsTest(unittest.TestCase):

    def test_get_body_spec_with_no_required_body_returns_empty_dict(self):
        schema = Schema()
        spec = schema.get_body_spec()
        self.assertEqual(spec, {'additionalProperties': False})


class RequestGapsTest(unittest.TestCase):

    def test_body_parser_exception_falls_back_to_raw_body(self):
        request = Request({
            'body': '<not valid xml',
            'headers': {'content-type': 'application/xml'},
        })
        self.assertEqual(request.body, '<not valid xml')

    def test_path_params_setter_recovers_when_internal_state_is_not_dict(self):
        request = Request({})
        request._Request__path_params = 'corrupted'  # pylint: disable=protected-access
        request.path_params = ('user_id', '42')
        self.assertEqual(request.path_params, {'user_id': '42'})


class OpenAPIGeneratorGapsTest(unittest.TestCase):

    def test_delete_unused_paths_removes_entire_path_when_not_registered(self):
        generator = OpenAPIGenerator('tests/mocks/apigateway/openapi/files/yml')
        self.assertIn('/unit-test/v1/schema', generator.doc['paths'])
        generator.delete_unused_paths()
        self.assertEqual(generator.doc['paths'], {})

    def test_delete_unused_paths_removes_only_unused_methods_within_kept_path(self):
        generator = OpenAPIGenerator('tests/mocks/apigateway/openapi/files/yml')
        generator._OpenAPIGenerator__existing_path_methods = {  # pylint: disable=protected-access
            '/unit-test/v1/schema': ['get'],
        }
        generator.delete_unused_paths()
        schema_path = generator.doc['paths']['/unit-test/v1/schema']
        self.assertIn('get', schema_path)
        self.assertNotIn('post', schema_path)
        self.assertNotIn('patch', schema_path)
        self.assertEqual(generator.doc['paths'], {
            '/unit-test/v1/schema': {'get': schema_path['get']},
        })

    def test_add_path_and_method_sets_security_and_request_and_response_bodies(self):
        importer = HandlerImporter()
        generator = OpenAPIGenerator('tests/mocks_outputs')
        modules = importer.get_modules_from_file_paths(
            ['tests/mocks/apigateway/openapi/coverage_gaps.py'],
            'tests/mocks/apigateway/openapi',
            'acai_aws/unit_test',
        )
        post_module = next(m for m in modules if m.method == 'post')
        generator.add_path_and_method(post_module)
        path = generator.doc['paths'][post_module.route_path]['post']
        self.assertEqual(path['security'], [{'AcaiGenerated': []}])
        self.assertIn('requestBody', path)
        self.assertEqual(
            path['requestBody']['content']['application/json']['schema']['$ref'],
            f'#/components/schemas/{post_module.request_body_schema_name}',
        )
        self.assertIn('responses', path)
        self.assertEqual(
            path['responses']['200']['content']['application/json']['schema']['$ref'],
            f'#/components/schemas/{post_module.response_body_schema_name}',
        )


class HandlerModuleGapsTest(unittest.TestCase):

    def test_request_body_schema_inlines_pydantic_defs(self):
        importer = HandlerImporter()
        modules = importer.get_modules_from_file_paths(
            ['tests/mocks/apigateway/openapi/coverage_gaps.py'],
            'tests/mocks/apigateway/openapi',
            'acai_aws/unit_test',
        )
        post_module = next(m for m in modules if m.method == 'post')
        schema = post_module.request_body_schema
        self.assertNotIn('$defs', schema)
        address_property = schema['properties']['address']
        self.assertEqual(address_property.get('type'), 'object')
        self.assertIn('street', address_property['properties'])
        self.assertIn('city', address_property['properties'])

    def test_schema_body_returns_none_for_non_pydantic_non_dict_schema(self):
        importer = HandlerImporter()
        modules = importer.get_modules_from_file_paths(
            ['tests/mocks/apigateway/openapi/coverage_gaps.py'],
            'tests/mocks/apigateway/openapi',
            'acai_aws/unit_test',
        )
        put_module = next(m for m in modules if m.method == 'put')
        self.assertIsNone(put_module.request_body_schema)


if __name__ == '__main__':
    unittest.main()



def get_basic():
    return {
        'eventSource': 'aws:kafka',
        'eventSourceArn': 'arn:aws:kafka:sa-east-1:123456789012:cluster/vpc-2priv-2pub/751d2973-a626-431c-9d4e-d7975eb44dd7-2',
        'bootstrapServers': 'b-2.demo-cluster-1.a1bcde.c1.kafka.us-east-1.amazonaws.com:9092,b-1.demo-cluster-1.a1bcde.c1.kafka.us-east-1.amazonaws.com:9092',
        'records': {
            'mytopic-0': [
                {
                    'topic': 'mytopic',
                    'partition': 0,
                    'offset': 15,
                    'timestamp': 1545084650987,
                    'timestampType': 'CREATE_TIME',
                    'key': 'c29tZV9rZXk=',
                    'value': 'eyJsYW5nIjogImVuLXVzIiwgInNtcyI6IHRydWUsICJlbWFpbCI6IHRydWUsICJwdXNoIjogdHJ1ZX0=',
                    'headers': [
                        {
                            'headerKey': [
                                104,
                                101,
                                97,
                                100,
                                101,
                                114,
                                86,
                                97,
                                108,
                                117,
                                101
                            ]
                        }
                    ]
                }
            ]
        }
    }



def get_basic():
    return {
        'Records': [
            {
                'EventVersion': '1.0',
                'EventSubscriptionArn': 'arn:aws:sns:us-east-1:123456789012:sns-lambda:21be56ed-a058-49f5-8c98-aedd2564c486',
                'EventSource': 'aws:sns',
                'Sns': {
                    'SignatureVersion': '1',
                    'Timestamp': '2019-01-02T12:45:07.000Z',
                    'Signature': 'tcc6faL2yUC6dgZdmrwh1Y4cGa/ebXEkAi6RibDsvpi+tE/1+82j...65r==',
                    'SigningCertUrl': 'https://sns.us-east-1.amazonaws.com/SimpleNotificationService-ac565b8b1a6c5d002d285f9598aa1d9b.pem',
                    'MessageId': '95df01b4-ee98-5cb9-9903-4c221d41eb5e',
                    'Message': '{"lang": "en-us", "sms": true, "email": true, "push": true}',
                    'MessageAttributes': {
                        'Test': {
                            'Type': 'String',
                            'Value': 'TestString'
                        },
                        'TestBinary': {
                            'Type': 'Binary',
                            'Value': 'TestBinary'
                        }
                    },
                    'Type': 'Notification',
                    'UnsubscribeUrl': 'https://sns.us-east-1.amazonaws.com/?Action=Unsubscribe&amp;SubscriptionArn=arn:aws:sns:us-east-1:123456789012:test-lambda:21be56ed-a058-49f5-8c98-aedd2564c486',
                    'TopicArn': 'arn:aws:sns:us-east-1:123456789012:sns-lambda',
                    'Subject': 'TestInvoke'
                }
            }
        ]
    }



import unittest
from unittest import mock

from acai_aws.apigateway.openapi.file_writer import OpenAPIFileWriter


def mock_open(*args, **kwargs):
    return mock.mock_open(read_data='bar')(*args, **kwargs)


class OpenAPIFileWriterTest(unittest.TestCase):

    def setUp(self):
        self.writer = OpenAPIFileWriter()

    @mock.patch('builtins.open', mock_open)
    def test_write_openapi(self):
        try:
            self.writer.write_openapi({'test': True}, 'tests/outputs/file_writer', ['json', 'yml'])
            self.assertTrue(True)
        except:
            self.assertTrue(False)



def get_created_event():
    return {
        'Records': [
            {
                'eventID': '9a37c0d03eb60f7cf70cabc823de9907',
                'eventName': 'INSERT',
                'eventVersion': '1.1',
                'eventSource': 'aws:dynamodb',
                'awsRegion': 'us-east-1',
                'dynamodb': {
                    'ApproximateCreationDateTime': 1538695200.0,
                    'Keys': {
                        'example_id': {
                            'S': '123456789'
                        }
                    },
                    'NewImage': {
                        'example_id': {
                            'S': '123456789'
                        },
                        'note': {
                            'S': 'Hosrawguw verrig zogupap ce so fajdis vub mos sif mawpowpug kif kihane.'
                        },
                        'active': {
                            'BOOL': True
                        },
                        'personal': {
                            'M': {
                                'gender': {
                                    'S': 'male'
                                },
                                'last_name': {
                                    'S': 'Mcneil'
                                },
                                'first_name': {
                                    'S': 'Mannix'
                                }
                            }
                        },
                        'transportation': {
                            'L': [
                                {
                                    'S': 'public-transit'
                                },
                                {
                                    'S': 'car-access'
                                }
                            ]
                        }
                    },
                    'SequenceNumber': '162100000000001439016707',
                    'SizeBytes': 1124,
                    'StreamViewType': 'NEW_AND_OLD_IMAGES'
                },
                'eventSourceARN': 'arn:aws:dynamodb:us-east-1:771875143460:table/test-example/stream/2019-10-04T23:18:26.340'
            }
        ]
    }


def get_updated_event():
    return {
        'Records': [
            {
                'eventID': '9a37c0d03eb60f7cf70cabc823de9907',
                'eventName': 'INSERT',
                'eventVersion': '1.1',
                'eventSource': 'aws:dynamodb',
                'awsRegion': 'us-east-1',
                'dynamodb': {
                    'ApproximateCreationDateTime': 1538695200.0,
                    'Keys': {
                        'example_id': {
                            'S': '123456789'
                        }
                    },
                    'OldImage': {
                        'example_id': {
                            'S': '123456789'
                        },
                        'note': {
                            'S': 'Hosrawguw verrig zogupap ce so fajdis vub mos sif mawpowpug kif kihane.'
                        },
                        'active': {
                            'BOOL': False
                        },
                        'personal': {
                            'M': {
                                'gender': {
                                    'S': 'male'
                                },
                                'last_name': {
                                    'S': 'Mcneil'
                                },
                                'first_name': {
                                    'S': 'Mannix'
                                }
                            }
                        },
                        'transportation': {
                            'L': [
                                {
                                    'S': 'public-transit'
                                },
                                {
                                    'S': 'car-access'
                                }
                            ]
                        }
                    },
                    'NewImage': {
                        'example_id': {
                            'S': '123456789'
                        },
                        'note': {
                            'S': 'Hosrawguw verrig zogupap ce so fajdis vub mos sif mawpowpug kif kihane.'
                        },
                        'active': {
                            'BOOL': True
                        },
                        'personal': {
                            'M': {
                                'gender': {
                                    'S': 'male'
                                },
                                'last_name': {
                                    'S': 'Mcneil'
                                },
                                'first_name': {
                                    'S': 'Mannix'
                                }
                            }
                        },
                        'transportation': {
                            'L': [
                                {
                                    'S': 'public-transit'
                                },
                                {
                                    'S': 'car-access'
                                }
                            ]
                        }
                    },
                    'SequenceNumber': '162100000000001439016707',
                    'SizeBytes': 1124,
                    'StreamViewType': 'NEW_AND_OLD_IMAGES'
                },
                'eventSourceARN': 'arn:aws:dynamodb:us-east-1:771875143460:table/test-example/stream/2019-10-04T23:18:26.340'
            }
        ]
    }


def get_deleted_event():
    return {
        'Records': [
            {
                'eventID': '9a37c0d03eb60f7cf70cabc823de9907',
                'eventName': 'INSERT',
                'eventVersion': '1.1',
                'eventSource': 'aws:dynamodb',
                'awsRegion': 'us-east-1',
                'dynamodb': {
                    'ApproximateCreationDateTime': 1538695200.0,
                    'Keys': {
                        'example_id': {
                            'S': '123456789'
                        }
                    },
                    'OldImage': {
                        'example_id': {
                            'S': '123456789'
                        },
                        'note': {
                            'S': 'Hosrawguw verrig zogupap ce so fajdis vub mos sif mawpowpug kif kihane.'
                        },
                        'active': {
                            'BOOL': True
                        },
                        'personal': {
                            'M': {
                                'gender': {
                                    'S': 'male'
                                },
                                'last_name': {
                                    'S': 'Mcneil'
                                },
                                'first_name': {
                                    'S': 'Mannix'
                                }
                            }
                        },
                        'transportation': {
                            'L': [
                                {
                                    'S': 'public-transit'
                                },
                                {
                                    'S': 'car-access'
                                }
                            ]
                        }
                    },
                    'SequenceNumber': '162100000000001439016707',
                    'SizeBytes': 1124,
                    'StreamViewType': 'NEW_AND_OLD_IMAGES'
                },
                'eventSourceARN': 'arn:aws:dynamodb:us-east-1:771875143460:table/test-example/stream/2019-10-04T23:18:26.340'
            }
        ]
    }



import unittest

from acai_aws.s3.record import Record

from tests.mocks.s3 import mock_event


class S3RecordTest(unittest.TestCase):
    basic_record = mock_event.get_basic()['Records'][0]
    removed_record = mock_event.get_basic_removed()['Records'][0]
    unknown_record = mock_event.get_basic_unknown()['Records'][0]

    def test_record_event_accepts_event(self):
        record = Record(self.basic_record)
        self.assertIsNone(record.body)
        self.assertEqual(record.name, self.basic_record['eventName'])
        self.assertEqual(record.source, self.basic_record['eventSource'])
        self.assertEqual(record.version, self.basic_record['eventVersion'])
        self.assertEqual(record.time, self.basic_record['eventTime'])
        self.assertEqual(record.region, self.basic_record['awsRegion'])
        self.assertEqual(record.request, self.basic_record['requestParameters'])
        self.assertEqual(record.response, self.basic_record['responseElements'])
        self.assertEqual(record.configuration_id, self.basic_record['s3']['configurationId'])
        self.assertEqual(record.object, self.basic_record['s3']['object'])
        self.assertEqual(record.bucket, self.basic_record['s3']['bucket']['name'])
        self.assertEqual(record.bucket_arn, self.basic_record['s3']['bucket']['arn'])
        self.assertEqual(record.bucket_owner, self.basic_record['s3']['bucket']['ownerIdentity']['principalId'])
        self.assertEqual(record.key, self.basic_record['s3']['object']['key'])
        self.assertEqual(record.schema_version, self.basic_record['s3']['s3SchemaVersion'])
        self.assertEqual(record.user_identity, self.basic_record['userIdentity']['principalId'])
        self.assertEqual(record.operation, record.CREATED)

    def test_record_event_accepts_remove_event(self):
        record = Record(self.removed_record)
        self.assertEqual(record.operation, record.DELETED)

    def test_record_event_accepts_unknown_event(self):
        record = Record(self.unknown_record)
        self.assertEqual(record.operation, record.UNKNOWN)

    def test_record_event_allows_body_to_be_set(self):
        expected = {'some_key': 'some_value'}
        record = Record(self.basic_record)
        record.body = expected.copy()
        self.assertDictEqual(record.body, expected)

    def test_record_event_prints(self):
        try:
            record = Record(self.basic_record)
            print(record)
            self.assertTrue(True)
        except Exception as error:
            print(error)
            self.assertTrue(False)



import base64

from acai_aws.apigateway.request import Request as ApigatewayRequest


class Request(ApigatewayRequest):

    def __init__(self, event, lambda_context=None, timeout=None):
        if event.get('isBase64Encoded') and event.get('body'):
            event = {
                **event,
                'body': base64.b64decode(event['body']).decode('utf-8'),
                'isBase64Encoded': False,
            }
        super().__init__(event, lambda_context, timeout)

    @property
    def target_group_arn(self):
        return self.event.get('requestContext', {}).get('elb', {}).get('targetGroupArn', '')

    @property
    def source_ip(self):
        return self.headers.get('x-forwarded-for', '')



import unittest

from acai_aws.alb.record import Record

from tests.mocks.alb import mock_event


class ALBRecordTest(unittest.TestCase):
    basic_event = mock_event.get_basic()
    base64_event = mock_event.get_base64()

    def test_record_body_parsed_from_json(self):
        record = Record(self.basic_event)
        self.assertEqual(record.body['business_id'], 'biz-e2e-001')
        self.assertEqual(record.body['decision'], 'APPROVED')
        self.assertEqual(record.body['completed'], '2026-03-30T12:40:40.353')

    def test_record_body_parsed_from_base64(self):
        record = Record(self.base64_event)
        self.assertEqual(record.body['business_id'], 'biz-e2e-002')
        self.assertEqual(record.body['decision'], 'DENIED')

    def test_record_operation_post_is_created(self):
        record = Record(mock_event.get_basic('POST'))
        self.assertEqual(record.operation, record.CREATED)

    def test_record_operation_put_is_updated(self):
        record = Record(mock_event.get_basic('PUT'))
        self.assertEqual(record.operation, record.UPDATED)

    def test_record_operation_patch_is_updated(self):
        record = Record(mock_event.get_basic('PATCH'))
        self.assertEqual(record.operation, record.UPDATED)

    def test_record_operation_delete_is_deleted(self):
        record = Record(mock_event.get_basic('DELETE'))
        self.assertEqual(record.operation, record.DELETED)

    def test_record_operation_get_is_unknown(self):
        record = Record(mock_event.get_basic('GET'))
        self.assertEqual(record.operation, record.UNKNOWN)

    def test_record_http_method(self):
        record = Record(self.basic_event)
        self.assertEqual(record.http_method, 'POST')

    def test_record_path(self):
        record = Record(self.basic_event)
        self.assertEqual(record.path, '/v1/kyc/callback')

    def test_record_headers(self):
        record = Record(self.basic_event)
        self.assertEqual(record.headers['content-type'], 'application/json')
        self.assertEqual(record.headers['host'], '10.84.140.68')

    def test_record_query_params(self):
        record = Record(self.basic_event)
        self.assertEqual(record.query_params['status'], 'approved')
        self.assertEqual(record.query_params['ref'], 'abc-123')

    def test_record_source_ip(self):
        record = Record(self.basic_event)
        self.assertEqual(record.source_ip, '10.104.8.48')

    def test_record_target_group_arn(self):
        record = Record(self.basic_event)
        self.assertEqual(
            record.target_group_arn,
            'arn:aws:elasticloadbalancing:us-east-1:123456789012:targetgroup/my-target-group/50dc6c495c0c9188'
        )

    def test_record_prints(self):
        try:
            record = Record(self.basic_event)
            print(record)
            self.assertTrue(True)
        except Exception as error:
            print(error)
            self.assertTrue(False)



from acai_aws.apigateway.requirements import requirements


@requirements(
    required_body='v1-resource-body'
)
def put(_, response):
    response.body = {'nested-2-resource': True}
    return response



import os


class InputValidator:

    def validate_arguments(self, input_args):
        self.__check_glob_pattern(input_args.handlers)
        self.__check_directory(input_args.output)

    def __check_glob_pattern(self, handlers):
        if '*.py' not in handlers:
            raise ValueError(f'{handlers} needs to be a glob pattern containing a "*.py" or valid directory location')

    def __check_directory(self, possible_dir):
        if not os.path.exists(possible_dir):
            raise ValueError(f'{possible_dir} is not a valid directory path')



from acai_aws.apigateway.requirements import requirements


@requirements(
    required_route='/nested_1/nested_2/{id}',
    required_body={
        'type': 'object',
        'required': [],
        'additionalProperties': False,
        'properties': {
            'grower_id': {'type': 'string'},
            'body': {'type': 'object'},
            'dict': {'type': 'boolean'}
        },
    }
)
def patch(_, response):
    response.body = {'nested-2-id': True}
    return response


@requirements(
    required_route='/nested_1/nested_2/{id}',
)
def delete(_, response):
    response.body = {'nested-2-id': False}
    return response



from acai_aws.apigateway.requirements import requirements

@requirements(
    required_body='v1-post-request-test'
)
def post(request, response):
    response.body = {'router_nested_mapping_basic': request.body}
    return response


@requirements(
    required_query=['auth_id'],
    available_query=['email', 'name']
)
def get(request, response):
    response.body = {'router_nested_mapping_basic': request.body}
    return response


@requirements(
    required_headers=['content-type'],
    available_headers=['correlation-id']
)
def delete(request, response):
    response.body = {'router_nested_mapping_basic': request.body}
    return response



from acai_aws.msk.requirements import requirements

call_list = []

class MockDataClass:
    def __init__(self, record):
        self.record = record
        self.initialized = True


def before_call(records, reqs):
    before_call.has_been_called = True
    global call_list
    call_list.append('before')


def after_call(records, result, reqs):
    after_call.has_been_called = True
    global call_list
    call_list.append('after')


@requirements(
    custom_requiremnent=True,
    before=before_call,
    after=after_call,
    data_class=MockDataClass
)
def mock_msk_full(event):
    full_results = []
    for mock_data_class in event.records:
        full_results.append(mock_data_class.initialized)
    return {'msk_full': full_results}



import importlib.util
import glob
import os

from acai_aws.apigateway.exception import ApiException


class ResolverImporter:

    def __init__(self, **kwargs):
        self.__handlers = self.clean_path(kwargs['handlers'])
        self.__handlers_tree = {}

    @staticmethod
    def import_module_from_file(file_path, import_path):
        spec = importlib.util.spec_from_file_location(import_path, file_path)
        handler_module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(handler_module)
        return handler_module

    @property
    def file_separator(self):
        return os.sep

    @property
    def handlers(self):
        return self.__handlers

    def clean_path(self, dirty_path):
        return dirty_path.strip(self.file_separator)

    def get_handlers_file_tree(self):
        if not self.__handlers_tree:
            glob_pattern = self.__get_glob_pattern()
            file_list = glob.glob(glob_pattern, recursive=True)
            file_paths = [item.replace(self.__get_handlers_root(), '') for item in file_list]
            for file_path in file_paths:
                sections = file_path.split(self.file_separator)
                sections = [section for section in sections if section]
                self.__recurse_section(self.__handlers_tree, sections, 0)
        return self.__handlers_tree

    def __get_glob_pattern(self):
        if '*' in self.__handlers and '.py' in self.__handlers:
            return self.handlers
        return self.handlers + self.file_separator + '**' + self.file_separator + '*.py'

    def __get_handlers_root(self):
        if '*' in self.__handlers and '.py' in self.__handlers:
            sep_split = self.handlers.split(self.file_separator)
            cleaned_split = [directory for directory in sep_split if self.__is_directory(directory)]
            return self.clean_path(f'{self.file_separator}'.join(cleaned_split))
        return self.handlers

    def __is_directory(self, directory):
        if '*' not in directory and '.py' not in directory:
            return True
        return False

    def __recurse_section(self, file_leaf, sections, index):
        if not index < len(sections):
            return
        section = sections[index]
        if not section: # pragma: no cover
            self.__recurse_section(file_leaf, sections, index + 1)
        if section not in file_leaf:
            file_leaf[section] = {} if index + 1 < len(sections) else '*'
        if isinstance(file_leaf, dict) and '__dynamic_files' not in file_leaf:
            file_leaf['__dynamic_files'] = set()
        if section.startswith('_') and not section.startswith('__'):
            file_leaf['__dynamic_files'].add(section)
        self.__check_multiple_dynamic_files(file_leaf, sections)
        self.__check_file_and_directory_share_name(file_leaf, section, sections)
        self.__recurse_section(file_leaf[section], sections, index + 1)

    def __check_multiple_dynamic_files(self, file_leaf, sections):
        if len(file_leaf['__dynamic_files']) > 1:
            files = ', '.join(list(file_leaf['__dynamic_files']))
            sections.pop()
            location = f'{self.file_separator}'.join(sections)
            raise ApiException(message=f'Cannot have two dynamic files in the same directory. Files: {files}, Location: {location}')

    def __check_file_and_directory_share_name(self, file_leaf, section, sections):
        opposite_type = section.replace('.py', '') if '.py' in section else f'{section}.py'
        if opposite_type in file_leaf:
            location = f'{self.file_separator}'.join(sections)
            raise ApiException(message=f'Cannot have file and directory share same name. Files: {section}, Location: {location}')



import json
import yaml


class OpenAPIFileWriter:

    def write_openapi(self, doc, file_location, formats):
        for write_format in formats:
            if write_format == 'json':
                self.__write_json(doc, file_location)
            if write_format == 'yml':
                self.__write_yml(doc, file_location)

    def __write_json(self, doc, file_locaiton):
        with open(f'{file_locaiton}/openapi.json', 'w', encoding='utf-8') as openapi_json:
            openapi_json.write(json.dumps(doc, indent=4))

    def __write_yml(self, doc, file_locaiton):
        with open(f'{file_locaiton}/openapi.yml', 'w', encoding='utf-8') as openapi_yml:
            yaml.dump(doc, openapi_yml, indent=4, default_flow_style=False, sort_keys=False)



import base64
import json


def _alb_request_context():
    return {
        'elb': {
            'targetGroupArn': 'arn:aws:elasticloadbalancing:us-east-1:123456789012:targetgroup/acai-tg/50dc6c495c0c9188'
        }
    }


def get_basic_post():
    return {
        'requestContext': _alb_request_context(),
        'httpMethod': 'POST',
        'path': 'unit-test/v1/basic',
        'queryStringParameters': {
            'name': 'me'
        },
        'headers': {
            'x-api-key': 'SOME-KEY',
            'content-type': 'application/json',
            'host': 'mock-alb.example.com',
            'x-forwarded-for': '203.0.113.42',
            'x-forwarded-port': '443',
            'x-forwarded-proto': 'https',
        },
        'body': json.dumps({'body_key': 'body_value'}),
        'isBase64Encoded': False,
    }


def get_basic_post_base64():
    event = get_basic_post()
    event['body'] = base64.b64encode(event['body'].encode('utf-8')).decode('utf-8')
    event['isBase64Encoded'] = True
    return event


def get_raised_exception_post():
    return {
        'requestContext': _alb_request_context(),
        'httpMethod': 'POST',
        'path': 'unit-test/v1/raise-exception',
        'queryStringParameters': {},
        'headers': {
            'content-type': 'application/json',
            'host': 'mock-alb.example.com',
            'x-forwarded-for': '203.0.113.42',
        },
        'body': json.dumps({}),
        'isBase64Encoded': False,
    }


def get_unhandled_exception_post():
    return {
        'requestContext': _alb_request_context(),
        'httpMethod': 'POST',
        'path': 'unit-test/v1/unhandled-exception',
        'queryStringParameters': {},
        'headers': {
            'content-type': 'application/json',
            'host': 'mock-alb.example.com',
            'x-forwarded-for': '203.0.113.42',
        },
        'body': json.dumps({}),
        'isBase64Encoded': False,
    }


def get_dynamic_event(headers=None, path='', method='get', body=None, query=None):
    event = {
        'requestContext': _alb_request_context(),
        'httpMethod': method.upper(),
        'path': path,
        'queryStringParameters': query if query is not None else {},
        'headers': headers if headers is not None else {'content-type': 'application/json'},
        'body': json.dumps(body) if body is not None else '',
        'isBase64Encoded': False,
    }
    event['headers'].setdefault('host', 'mock-alb.example.com')
    return event



def get_basic_removed():
    return {
        'Records': [
            {
                'eventVersion': '2.0',
                'eventSource': 'aws:s3',
                'awsRegion': 'us-east-1',
                'eventTime': '2018-09-20T21: 10: 13.821Z',
                'eventName': 'ObjectRemoved: Put',
                'userIdentity': {
                    'principalId': 'AWS: AROAI7Z5ZQEQ3UETKKYGQ: deploy-workers-poc-put-v1-photo'
                },
                'requestParameters': {
                    'sourceIPAddress': '172.20.133.36'
                },
                'responseElements': {
                    'x-amz-request-id': '6B859DD0CE613FAE',
                    'x-amz-id-2': 'EXLMfc9aiXZFzNwLKXpw35iaVvl/DkEA6GtbuxjfmuLN3kLPL/aGoa7NMSwpl3m7ICAtNbjJX4w='
                },
                's3': {
                    's3SchemaVersion': '1.0',
                    'configurationId': 'exS3-v2--7cde234c7ff76c53c44990396aeddc6d',
                    'bucket': {
                        'name': 'user-preferences',
                        'ownerIdentity': {
                            'principalId': 'A32KFL0DQ3MH8X'
                        },
                        'arn': 'arn:aws:s3:::user-preferences'
                    },
                    'object': {
                        'key': 'user-1-prefs.json',
                        'size': 17545,
                        'eTag': 'b79ac2ef68c08fa9ac6013d53038a26c',
                        'sequencer': '005BA40CB5BD42013A'
                    }
                }
            }
        ]
    }

def get_basic_unknown():
    return {
        'Records': [
            {
                'eventVersion': '2.0',
                'eventSource': 'aws:s3',
                'awsRegion': 'us-east-1',
                'eventTime': '2018-09-20T21: 10: 13.821Z',
                'eventName': 'SomethingElse: Put',
                'userIdentity': {
                    'principalId': 'AWS: AROAI7Z5ZQEQ3UETKKYGQ: deploy-workers-poc-put-v1-photo'
                },
                'requestParameters': {
                    'sourceIPAddress': '172.20.133.36'
                },
                'responseElements': {
                    'x-amz-request-id': '6B859DD0CE613FAE',
                    'x-amz-id-2': 'EXLMfc9aiXZFzNwLKXpw35iaVvl/DkEA6GtbuxjfmuLN3kLPL/aGoa7NMSwpl3m7ICAtNbjJX4w='
                },
                's3': {
                    's3SchemaVersion': '1.0',
                    'configurationId': 'exS3-v2--7cde234c7ff76c53c44990396aeddc6d',
                    'bucket': {
                        'name': 'user-preferences',
                        'ownerIdentity': {
                            'principalId': 'A32KFL0DQ3MH8X'
                        },
                        'arn': 'arn:aws:s3:::user-preferences'
                    },
                    'object': {
                        'key': 'user-1-prefs.json',
                        'size': 17545,
                        'eTag': 'b79ac2ef68c08fa9ac6013d53038a26c',
                        'sequencer': '005BA40CB5BD42013A'
                    }
                }
            }
        ]
    }


def get_basic():
    return {
        'Records': [
            {
                'eventVersion': '2.0',
                'eventSource': 'aws:s3',
                'awsRegion': 'us-east-1',
                'eventTime': '2018-09-20T21: 10: 13.821Z',
                'eventName': 'ObjectCreated: Put',
                'userIdentity': {
                    'principalId': 'AWS: AROAI7Z5ZQEQ3UETKKYGQ: deploy-workers-poc-put-v1-photo'
                },
                'requestParameters': {
                    'sourceIPAddress': '172.20.133.36'
                },
                'responseElements': {
                    'x-amz-request-id': '6B859DD0CE613FAE',
                    'x-amz-id-2': 'EXLMfc9aiXZFzNwLKXpw35iaVvl/DkEA6GtbuxjfmuLN3kLPL/aGoa7NMSwpl3m7ICAtNbjJX4w='
                },
                's3': {
                    's3SchemaVersion': '1.0',
                    'configurationId': 'exS3-v2--7cde234c7ff76c53c44990396aeddc6d',
                    'bucket': {
                        'name': 'user-preferences',
                        'ownerIdentity': {
                            'principalId': 'A32KFL0DQ3MH8X'
                        },
                        'arn': 'arn:aws:s3:::user-preferences'
                    },
                    'object': {
                        'key': 'user-1-prefs.json',
                        'size': 17545,
                        'eTag': 'b79ac2ef68c08fa9ac6013d53038a26c',
                        'sequencer': '005BA40CB5BD42013A'
                    }
                }
            }
        ]
    }


def get_basic_csv():
    return {
        'Records': [
            {
                'eventVersion': '2.0',
                'eventSource': 'aws:s3',
                'awsRegion': 'us-east-1',
                'eventTime': '2018-09-20T21: 10: 13.821Z',
                'eventName': 'ObjectCreated: Put',
                'userIdentity': {
                    'principalId': 'AWS:AROAI7Z5ZQEQ3UETKKYGQ:user-preferences'
                },
                'requestParameters': {
                    'sourceIPAddress': '172.20.133.36'
                },
                'responseElements': {
                    'x-amz-request-id': '6B859DD0CE613FAE',
                    'x-amz-id-2': 'EXLMfc9aiXZFzNwLKXpw35iaVvl/DkEA6GtbuxjfmuLN3kLPL/aGoa7NMSwpl3m7ICAtNbjJX4w='
                },
                's3': {
                    's3SchemaVersion': '1.0',
                    'configurationId': 'exS3-v2--7cde234c7ff76c53c44990396aeddc6d',
                    'bucket': {
                        'name': 'user-preferences',
                        'ownerIdentity': {
                            'principalId': 'A32KFL0DQ3MH8X'
                        },
                        'arn': 'arn:aws:s3:::user-preferences'
                    },
                    'object': {
                        'key': 'users.csv',
                        'size': 17545,
                        'eTag': 'b79ac2ef68c08fa9ac6013d53038a26c',
                        'sequencer': '005BA40CB5BD42013A'
                    }
                }
            }
        ]
    }




import unittest

from acai_aws.common.records.exception import RecordException

from tests.mocks.s3 import mock_event
from tests.mocks.s3.mock_functions import mock_s3_full, mock_s3_basic, mock_s3_operation_ignore, mock_s3_operation_raise, before_call, after_call, call_list


class S3RequirementsTest(unittest.TestCase):
    basic_event = mock_event.get_basic()

    def test_s3_decorator_with_basic_requirements(self):
        expected = {'s3_basic': ['ObjectCreated: Put']}
        result = mock_s3_basic(self.basic_event, None)
        self.assertDictEqual(result, expected)

    def test_s3_decorator_with_full_requirements(self):
        expected = {'s3_full': [True]}
        result = mock_s3_full(self.basic_event, None)
        self.assertDictEqual(result, expected)
        self.assertTrue(before_call.has_been_called)
        self.assertTrue(after_call.has_been_called)
        self.assertEqual('before', call_list[0])
        self.assertEqual('after', call_list[1])

    def test_s3_decorator_ignore_non_matching_operation(self):
        expected = {'s3_operation_ignore': []}
        result = mock_s3_operation_ignore(self.basic_event, None)
        self.assertDictEqual(result, expected)

    def test_s3_decorator_ignore_non_matching_operation_error(self):
        try:
            mock_s3_operation_raise(self.basic_event, None)
            self.assertTrue(False)
        except RecordException as record_error:
            self.assertTrue(isinstance(record_error, RecordException))



from acai_aws.apigateway.requirements import requirements

@requirements(
    required_headers=['content-type'],
    required_route='/nested/{nested_id}',
    required_body='v1-patch-request-test'
)
def patch(request, response):
    response.body = {'router_nested_directory_dynamic': request.body, 'path_params': request.path_params}
    return response
